# Copyright: Ankitects Pty Ltd and contributors
# License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html
from __future__ import annotations

import time
from collections.abc import Callable
from concurrent.futures import Future
from dataclasses import dataclass

import aqt.forms
from anki._legacy import print_deprecation_warning
from anki.collection import Progress
from aqt.qt import *
from aqt.qt import sip
from aqt.utils import disable_help_button, tr

# Progress info
##########################################################################


class ProgressManager:
    def __init__(self, mw: aqt.AnkiQt) -> None:
        self.mw = mw
        self.app = mw.app
        self.inDB = False
        self.blockUpdates = False
        self._show_timer: QTimer | None = None
        self._busy_cursor_timer: QTimer | None = None
        self._win: ProgressDialog | None = None
        self._levels = 0
        self._backend_timer: QTimer | None = None

    # Safer timers
    ##########################################################################
    # Custom timers which avoid firing while a progress dialog is active
    # (likely due to some long-running DB operation)

    def timer(
        self,
        ms: int,
        func: Callable,
        repeat: bool,
        requiresCollection: bool = True,
        *,
        parent: QObject | None = None,
    ) -> QTimer:
        """Create and start a standard Anki timer. For an alternative see `single_shot()`.

        If the timer fires while a progress window is shown:
        - if it is a repeating timer, it will wait the same delay again
        - if it is non-repeating, it will try again in 100ms

        If requiresCollection is True, the timer will not fire if the
        collection has been unloaded. Setting it to False will allow the
        timer to fire even when there is no collection, but will still
        only fire when there is no current progress dialog.


        Issues and alternative
        ---
        The created timer will only be destroyed when `parent` is destroyed.
        This can cause memory leaks, because anything captured by `func` isn't freed either.
        If there is no QObject that will get destroyed reasonably soon, and you have to
        pass `mw`, you should call `deleteLater()` on the returned QTimer as soon as
        it's served its purpose, or use `single_shot()`.

        Also note that you may not be able to pass an adequate parent, if you want to
        make a callback after a widget closes. If you passed that widget, the timer
        would get destroyed before it could fire.
        """

        if parent is None:
            print_deprecation_warning(
                "to avoid memory leaks, pass an appropriate parent to progress.timer()"
                " or use progress.single_shot()"
            )
            parent = self.mw

        qtimer = QTimer(parent)
        if not repeat:
            qtimer.setSingleShot(True)
        qconnect(qtimer.timeout, self._get_handler(func, repeat, requiresCollection))
        qtimer.start(ms)
        return qtimer

    def single_shot(
        self,
        ms: int,
        func: Callable[[], None],
        requires_collection: bool = True,
    ) -> None:
        """Create and start a one-off Anki timer. For an alternative and more
        documentation, see `timer()`.


        Issues and alternative
        ---
        `single_shot()` cleans itself up, so a passed closure won't leak any memory.
        However, if `func` references a QObject other than `mw`, which gets deleted before the
        timer fires, an Exception is raised. To avoid this, either use `timer()` passing
        that object as the parent, or check in `func` with `sip.isdeleted(object)` if
        it still exists.

        On the other hand, if a widget is supposed to make an external callback after it closes,
        you likely want to use `single_shot()`, which will fire even if the calling
        widget is already destroyed.
        """
        QTimer.singleShot(ms, self._get_handler(func, False, requires_collection))

    def _get_handler(
        self, func: Callable[[], None], repeat: bool, requires_collection: bool
    ) -> Callable[[], None]:
        def handler() -> None:
            if requires_collection and not self.mw.col:
                # no current collection; timer is no longer valid
                print(f"Ignored progress func as collection unloaded: {repr(func)}")
                return

            if not self._levels:
                # no current progress; safe to fire
                func()
            elif repeat:
                # skip this time; we'll fire again
                pass
            else:
                # retry in 100ms
                self.single_shot(100, func, requires_collection)

        return handler

    # Creating progress dialogs
    ##########################################################################

    def start(
        self,
        max: int = 0,
        min: int = 0,
        label: str | None = None,
        parent: QWidget | None = None,
        immediate: bool = False,
        title: str = "Anki",
    ) -> ProgressDialog | None:
        self._levels += 1
        if self._levels > 1:
            return None
        # setup window
        parent = parent or self.app.activeWindow()
        if not parent and self.mw.isVisible():
            parent = self.mw

        label = label or tr.qt_misc_processing()
        self._win = ProgressDialog(parent)
        self._win.form.progressBar.setMinimum(min)
        self._win.form.progressBar.setMaximum(max)
        self._win.form.progressBar.setTextVisible(False)
        self._win.form.label.setText(label)
        self._win.setWindowTitle(title)
        self._win.setWindowModality(Qt.WindowModality.ApplicationModal)
        self._win.setMinimumWidth(300)
        self._busy_cursor_timer = QTimer(self.mw)
        self._busy_cursor_timer.setSingleShot(True)
        self._busy_cursor_timer.start(300)
        qconnect(self._busy_cursor_timer.timeout, self._set_busy_cursor)
        self._shown: float = 0
        self._counter = min
        self._min = min
        self._max = max
        self._firstTime = time.monotonic()
        self._show_timer = QTimer(self.mw)
        self._show_timer.setSingleShot(True)
        self._show_timer.start(immediate and 100 or 600)
        qconnect(self._show_timer.timeout, self._on_show_timer)
        return self._win

    def start_with_backend_updates(
        self,
        progress_update: Callable[[Progress, ProgressUpdate], None],
        start_label: str | None = None,
        parent: QWidget | None = None,
    ) -> None:
        self._backend_timer = QTimer()
        self._backend_timer.setSingleShot(False)
        self._backend_timer.setInterval(100)

        if not (dialog := self.start(immediate=True, label=start_label, parent=parent)):
            print("Progress dialog already running; aborting will not work")

        def on_progress() -> None:
            assert self.mw

            user_wants_abort = dialog and dialog.wantCancel or False
            update = ProgressUpdate(user_wants_abort=user_wants_abort)
            progress = self.mw.backend.latest_progress()
            progress_update(progress, update)
            if update.abort:
                self.mw.backend.set_wants_abort()
            if update.has_update():
                self.update(label=update.label, value=update.value, max=update.max)

        qconnect(self._backend_timer.timeout, on_progress)
        self._backend_timer.start()

    def update(
        self,
        label: str | None = None,
        value: int | None = None,
        process: bool = True,
        maybeShow: bool = True,
        max: int | None = None,
    ) -> None:
        # print("update", label, self._levels, self._min, self._counter, self._max, label, time.monotonic() - self._shown)
        if not self.mw.inMainThread():
            print("progress.update() called on wrong thread")
            return
        if maybeShow:
            self._maybeShow()
        if not self._shown:
            return

        assert self._win is not None
        if label:
            self._win.form.label.setText(label)

        self._max = max or 0
        self._win.form.progressBar.setMaximum(self._max)
        if self._max:
            self._counter = value if value is not None else (self._counter + 1)
            self._win.form.progressBar.setValue(self._counter)

    def finish(self) -> None:
        def do_window_cleanup(future: Future | None = None):
            # this method can be called in an async fashion from taskman where a future
            # is passed or in synchronous manner from the main thread
            if future is not None:
                future.result()

            next_levels = self._levels - 1
            next_levels = max(0, next_levels)
            try:
                if next_levels == 0:
                    if self._win:
                        self._closeWin()
                    if self._busy_cursor_timer:
                        self._busy_cursor_timer.stop()
                        self._busy_cursor_timer = None
                    self._restore_cursor()
                    if self._show_timer:
                        self._show_timer.stop()
                        self._show_timer = None
                if self._backend_timer:
                    self._backend_timer.stop()
                    self._backend_timer.deleteLater()
                    self._backend_timer = None
            except RuntimeError as exc:
                # during shutdown, the timers may have already been deleted by Qt
                print(f"do_window_cleanup error ignored: {exc}")
            self._levels = next_levels

        # if the window is not currently shown, we can do cleanup immediately, if it is
        # currently shown then we need to give the window system a half-second to
        # present the window before we close it again - fixes progress window getting
        # stuck, especially on ubuntu 16.10+
        elapsed_time = time.monotonic() - self._shown
        time_to_wait = 0.5 - elapsed_time
        # NOTE: we must not yield control if the window is not shown since we don't want
        # to expose ourselves to the possibility of something showing the window in the
        # meantime
        if (not self._shown) or (time_to_wait <= 0):
            do_window_cleanup()
        else:
            # NOTE: we can't use self.single_shot here since it won't call the callback
            # until _levels = 0, but if we are in this branch then _levels > 0
            self.mw.taskman.run_in_background(
                lambda time_to_wait=time_to_wait: time.sleep(time_to_wait),
                do_window_cleanup,
                uses_collection=False,
            )

    def clear(self) -> None:
        "Restore the interface after an error."
        if self._levels:
            self._levels = 1
            self.finish()

    def _maybeShow(self) -> None:
        if not self._levels:
            return
        if self._shown:
            return
        delta = time.monotonic() - self._firstTime
        if delta > 0.5:
            self._showWin()

    def _showWin(self) -> None:
        assert self._win is not None
        self._shown = time.monotonic()
        self._win.show()

    def _closeWin(self) -> None:
        # if the parent window has been deleted, the progress dialog may have
        # already been dropped; delete it if it hasn't been
        if self._win and not sip.isdeleted(self._win):
            self._win.cancel()
        self._win = None
        self._shown = 0

    def _set_busy_cursor(self) -> None:
        self.mw.app.setOverrideCursor(QCursor(Qt.CursorShape.WaitCursor))

    def _restore_cursor(self) -> None:
        self.app.restoreOverrideCursor()

    def busy(self) -> int:
        "True if processing."
        return self._levels

    def _on_show_timer(self) -> None:
        if self.mw.app.focusWindow() is None:
            # if no window is focused (eg app is minimized), defer display
            assert self._show_timer is not None
            self._show_timer.start(10)
            return

        self._show_timer = None
        self._showWin()

    def want_cancel(self) -> bool:
        win = self._win
        if win:
            return win.wantCancel
        else:
            return False

    def set_title(self, title: str) -> None:
        win = self._win
        if win:
            win.setWindowTitle(title)


class ProgressDialog(QDialog):
    def __init__(self, parent: QWidget | None) -> None:
        QDialog.__init__(self, parent)
        disable_help_button(self)
        self.form = aqt.forms.progress.Ui_Dialog()
        self.form.setupUi(self)
        self._closingDown = False
        self.wantCancel = False
        # required for smooth progress bars
        self.form.progressBar.setStyleSheet("QProgressBar::chunk { width: 1px; }")

    def cancel(self) -> None:
        self._closingDown = True
        self.hide()
        self.deleteLater()

    def closeEvent(self, evt: QCloseEvent | None) -> None:
        assert evt is not None
        if self._closingDown:
            evt.accept()
        else:
            self.wantCancel = True
            evt.ignore()

    def keyPressEvent(self, evt: QKeyEvent | None) -> None:
        assert evt is not None
        if evt.key() == Qt.Key.Key_Escape:
            evt.ignore()
            self.wantCancel = True


@dataclass
class ProgressUpdate:
    label: str | None = None
    value: int | None = None
    max: int | None = None
    user_wants_abort: bool = False
    abort: bool = False

    def has_update(self) -> bool:
        return self.label is not None or self.value is not None or self.max is not None



# Copyright: Ankitects Pty Ltd and contributors
# License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html


from __future__ import annotations

from typing import TYPE_CHECKING, Any

from anki.notes import NoteId

if TYPE_CHECKING:
    from anki.collection import Collection


class Finder:
    def __init__(self, col: Collection | None) -> None:
        self.col = col.weakref()
        print("Finder() is deprecated, please use col.find_cards() or .find_notes()")

    def findCards(self, query: Any, order: Any) -> Any:
        return self.col.find_cards(query, order)

    def findNotes(self, query: Any) -> Any:
        return self.col.find_notes(query)


# Find and replace
##########################################################################


def findReplace(
    col: Collection,
    nids: list[NoteId],
    src: str,
    dst: str,
    regex: bool = False,
    field: str | None = None,
    fold: bool = True,
) -> int:
    "Find and replace fields in a note. Returns changed note count."
    print("use col.find_and_replace() instead of findReplace()")
    return col.find_and_replace(
        note_ids=nids,
        search=src,
        replacement=dst,
        regex=regex,
        match_case=not fold,
        field_name=field,
    ).count


def fieldNamesForNotes(col: Collection, nids: list[NoteId]) -> list[str]:
    return list(col.field_names_for_note_ids(nids))


# Find duplicates
##########################################################################


def fieldNames(col: Collection, downcase: bool = True) -> list[str]:
    fields: set[str] = set()
    for m in col.models.all():
        for f in m["flds"]:
            name = f["name"].lower() if downcase else f["name"]
            if name not in fields:  # slower w/o
                fields.add(name)
    return list(fields)



# Copyright: Ankitects Pty Ltd and contributors
# License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html

from __future__ import annotations

import os
import re
from abc import ABC, abstractmethod
from collections.abc import Callable
from itertools import chain

import aqt.main
from anki.collection import Collection, Progress
from anki.errors import Interrupted
from anki.foreign_data import mnemosyne
from anki.lang import without_unicode_isolation
from anki.utils import tmpdir
from aqt.import_export.import_dialog import (
    AnkiPackageArgs,
    CsvArgs,
    ImportDialog,
    JsonFileArgs,
)
from aqt.operations import QueryOp
from aqt.progress import ProgressUpdate
from aqt.qt import *
from aqt.utils import askUser, getFile, showWarning, tooltip, tr


class Importer(ABC):
    accepted_file_endings: list[str]

    @classmethod
    def can_import(cls, lowercase_filename: str) -> bool:
        return any(
            lowercase_filename.endswith(ending) for ending in cls.accepted_file_endings
        )

    @classmethod
    @abstractmethod
    def do_import(cls, mw: aqt.main.AnkiQt, path: str) -> None: ...


class ColpkgImporter(Importer):
    accepted_file_endings = [".apkg", ".colpkg"]

    @staticmethod
    def can_import(filename: str) -> bool:
        return (
            filename == "collection.apkg"
            or (filename.startswith("backup-") and filename.endswith(".apkg"))
            or filename.endswith(".colpkg")
        )

    @staticmethod
    def do_import(mw: aqt.main.AnkiQt, path: str) -> None:
        if askUser(
            tr.importing_this_will_delete_your_existing_collection(),
            msgfunc=QMessageBox.warning,
            defaultno=True,
        ):
            ColpkgImporter._import(mw, path)

    @staticmethod
    def _import(mw: aqt.main.AnkiQt, file: str) -> None:
        def on_success() -> None:
            mw.loadCollection()
            tooltip(tr.importing_importing_complete())

        def on_failure(err: Exception) -> None:
            mw.loadCollection()
            if not isinstance(err, Interrupted):
                showWarning(str(err))

        QueryOp(
            parent=mw,
            op=lambda _: mw.create_backup_now(),
            success=lambda _: mw.unloadCollection(
                lambda: import_collection_package_op(mw, file, on_success)
                .failure(on_failure)
                .run_in_background()
            ),
        ).with_progress().run_in_background()


class ApkgImporter(Importer):
    accepted_file_endings = [".apkg", ".zip"]

    @staticmethod
    def do_import(mw: aqt.main.AnkiQt, path: str) -> None:
        ImportDialog(mw, AnkiPackageArgs(path))


class MnemosyneImporter(Importer):
    accepted_file_endings = [".db"]

    @staticmethod
    def do_import(mw: aqt.main.AnkiQt, path: str) -> None:
        def on_success(json: str) -> None:
            json_path = os.path.join(tmpdir(), os.path.basename(path))
            with open(json_path, "wb") as file:
                file.write(json.encode("utf8"))
            ImportDialog(mw, JsonFileArgs(path=json_path))

        QueryOp(
            parent=mw,
            op=lambda col: mnemosyne.serialize(path, col.decks.current()["id"]),
            success=on_success,
        ).with_progress().run_in_background()


class CsvImporter(Importer):
    accepted_file_endings = [".csv", ".tsv", ".txt"]

    @staticmethod
    def do_import(mw: aqt.main.AnkiQt, path: str) -> None:
        ImportDialog(mw, CsvArgs(path))


class JsonImporter(Importer):
    accepted_file_endings = [".anki-json"]

    @staticmethod
    def do_import(mw: aqt.main.AnkiQt, path: str) -> None:
        ImportDialog(mw, JsonFileArgs(path=path))


IMPORTERS: list[type[Importer]] = [
    ColpkgImporter,
    ApkgImporter,
    MnemosyneImporter,
    CsvImporter,
]


def legacy_file_endings(col: Collection) -> list[str]:
    from anki.importing import AnkiPackageImporter, TextImporter, importers
    from anki.importing import MnemosyneImporter as LegacyMnemosyneImporter

    return [
        ext
        for (text, importer) in importers(col)
        if importer not in (TextImporter, AnkiPackageImporter, LegacyMnemosyneImporter)
        for ext in re.findall(r"[( ]?\*(\..+?)[) ]", text)
    ]


def import_file(mw: aqt.main.AnkiQt, path: str) -> None:
    filename = os.path.basename(path).lower()

    if any(filename.endswith(ext) for ext in legacy_file_endings(mw.col)):
        import aqt.importing

        aqt.importing.importFile(mw, path)
        return

    for importer in IMPORTERS:
        if importer.can_import(filename):
            importer.do_import(mw, path)
            return

    showWarning("Unsupported file type.")


def prompt_for_file_then_import(mw: aqt.main.AnkiQt) -> None:
    if path := get_file_path(mw):
        import_file(mw, path)


def get_file_path(mw: aqt.main.AnkiQt) -> str | None:
    filter = without_unicode_isolation(
        tr.importing_all_supported_formats(
            val="({})".format(
                " ".join(f"*{ending}" for ending in all_accepted_file_endings(mw))
            )
        )
    )
    if file := getFile(mw, tr.actions_import(), None, key="import", filter=filter):
        return str(file)
    return None


def all_accepted_file_endings(mw: aqt.main.AnkiQt) -> set[str]:
    return set(
        chain(
            *(importer.accepted_file_endings for importer in IMPORTERS),
            legacy_file_endings(mw.col),
        )
    )


def import_collection_package_op(
    mw: aqt.main.AnkiQt, path: str, success: Callable[[], None]
) -> QueryOp[None]:
    def op(_: Collection) -> None:
        col_path = mw.pm.collectionPath()
        media_folder = os.path.join(mw.pm.profileFolder(), "collection.media")
        media_db = os.path.join(mw.pm.profileFolder(), "collection.media.db2")
        mw.backend.import_collection_package(
            col_path=col_path,
            backup_path=path,
            media_folder=media_folder,
            media_db=media_db,
        )

    return QueryOp(parent=mw, op=op, success=lambda _: success()).with_backend_progress(
        import_progress_update
    )


def import_progress_update(progress: Progress, update: ProgressUpdate) -> None:
    if not progress.HasField("importing"):
        return
    update.label = progress.importing
    if update.user_wants_abort:
        update.abort = True



# Copyright: Ankitects Pty Ltd and contributors
# License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html

from __future__ import annotations

from collections.abc import Sequence

from anki.cards import CardId
from anki.collection import OpChangesWithCount
from anki.decks import DeckId
from aqt.operations import CollectionOp
from aqt.qt import QWidget
from aqt.utils import tooltip, tr


def set_card_deck(
    *, parent: QWidget, card_ids: Sequence[CardId], deck_id: DeckId
) -> CollectionOp[OpChangesWithCount]:
    return CollectionOp(parent, lambda col: col.set_deck(card_ids, deck_id)).success(
        lambda out: tooltip(tr.browsing_cards_updated(count=out.count), parent=parent)
    )


def set_card_flag(
    *,
    parent: QWidget,
    card_ids: Sequence[CardId],
    flag: int,
) -> CollectionOp[OpChangesWithCount]:
    return CollectionOp(
        parent, lambda col: col.set_user_flag_for_cards(flag, card_ids)
    ).success(
        lambda out: tooltip(tr.browsing_cards_updated(count=out.count), parent=parent)
    )



# Copyright: Ankitects Pty Ltd and contributors
# License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html

from __future__ import annotations

from enum import Enum

from aqt.qt import QEvent, QObject, QSplitter, Qt


class BrowserLayout(Enum):
    AUTO = "auto"
    VERTICAL = "vertical"
    HORIZONTAL = "horizontal"


class QSplitterHandleEventFilter(QObject):
    """Event filter that equalizes QSplitter panes on double-clicking the handle"""

    def __init__(self, splitter: QSplitter):
        super().__init__(splitter)
        self._splitter = splitter

    def eventFilter(self, object: QObject | None, event: QEvent | None) -> bool:
        assert event is not None

        if event.type() == QEvent.Type.MouseButtonDblClick:
            splitter_parent = self._splitter.parentWidget()

            assert splitter_parent is not None

            if self._splitter.orientation() == Qt.Orientation.Horizontal:
                half_size = splitter_parent.width() // 2
            else:
                half_size = splitter_parent.height() // 2
            self._splitter.setSizes([half_size, half_size])
            return True

        return super().eventFilter(object, event)



# Copyright: Ankitects Pty Ltd and contributors
# License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html

from __future__ import annotations

import aqt
import aqt.browser
import aqt.gui_hooks
from aqt.qt import *


class SidebarSearchBar(QLineEdit):
    def __init__(self, sidebar: aqt.browser.sidebar.SidebarTreeView) -> None:
        QLineEdit.__init__(self, sidebar)
        self.setPlaceholderText(sidebar.col.tr.browsing_sidebar_filter())
        self.sidebar = sidebar
        self.timer = QTimer(self)
        self.timer.setInterval(600)
        self.timer.setSingleShot(True)
        self.setFrame(False)

        qconnect(self.timer.timeout, self.onSearch)
        qconnect(self.textChanged, self.onTextChanged)

    def onTextChanged(self, text: str) -> None:
        if not self.timer.isActive():
            self.timer.start()

    def onSearch(self) -> None:
        self.sidebar.search_for(self.text())

    def keyPressEvent(self, evt: QKeyEvent | None) -> None:
        assert evt is not None
        if evt.key() in (Qt.Key.Key_Up, Qt.Key.Key_Down):
            self.sidebar.setFocus()
        elif evt.key() in (Qt.Key.Key_Enter, Qt.Key.Key_Return):
            self.onSearch()
        else:
            QLineEdit.keyPressEvent(self, evt)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: anki/deck_config.proto
# Protobuf Python Version: 7.34.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    7,
    34,
    1,
    '',
    'anki/deck_config.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from anki import generic_pb2 as anki_dot_generic__pb2
from anki import collection_pb2 as anki_dot_collection__pb2
from anki import decks_pb2 as anki_dot_decks__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16\x61nki/deck_config.proto\x12\x10\x61nki.deck_config\x1a\x12\x61nki/generic.proto\x1a\x15\x61nki/collection.proto\x1a\x10\x61nki/decks.proto\"\x1c\n\x0c\x44\x65\x63kConfigId\x12\x0c\n\x04\x64\x63id\x18\x01 \x01(\x03\"8\n\x1bGetRetentionWorkloadRequest\x12\t\n\x01w\x18\x01 \x03(\x02\x12\x0e\n\x06search\x18\x02 \x01(\t\"\x96\x01\n\x1cGetRetentionWorkloadResponse\x12H\n\x05\x63osts\x18\x01 \x03(\x0b\x32\x39.anki.deck_config.GetRetentionWorkloadResponse.CostsEntry\x1a,\n\nCostsEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12\r\n\x05value\x18\x02 \x01(\x02:\x02\x38\x01\"R\n\x1cGetIgnoredBeforeCountRequest\x12\"\n\x1aignore_revlogs_before_date\x18\x01 \x01(\t\x12\x0e\n\x06search\x18\x02 \x01(\t\"@\n\x1dGetIgnoredBeforeCountResponse\x12\x10\n\x08included\x18\x01 \x01(\x04\x12\r\n\x05total\x18\x02 \x01(\x04\"\xf1\x19\n\nDeckConfig\x12\n\n\x02id\x18\x01 \x01(\x03\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x12\n\nmtime_secs\x18\x03 \x01(\x03\x12\x0b\n\x03usn\x18\x04 \x01(\x05\x12\x33\n\x06\x63onfig\x18\x05 \x01(\x0b\x32#.anki.deck_config.DeckConfig.Config\x1a\xf2\x18\n\x06\x43onfig\x12\x13\n\x0blearn_steps\x18\x01 \x03(\x02\x12\x15\n\rrelearn_steps\x18\x02 \x03(\x02\x12\x15\n\rfsrs_params_4\x18\x03 \x03(\x02\x12\x15\n\rfsrs_params_5\x18\x05 \x03(\x02\x12\x15\n\rfsrs_params_6\x18\x06 \x03(\x02\x12\x13\n\x0bnew_per_day\x18\t \x01(\r\x12\x17\n\x0freviews_per_day\x18\n \x01(\r\x12\x1b\n\x13new_per_day_minimum\x18# \x01(\r\x12\x14\n\x0cinitial_ease\x18\x0b \x01(\x02\x12\x17\n\x0f\x65\x61sy_multiplier\x18\x0c \x01(\x02\x12\x17\n\x0fhard_multiplier\x18\r \x01(\x02\x12\x18\n\x10lapse_multiplier\x18\x0e \x01(\x02\x12\x1b\n\x13interval_multiplier\x18\x0f \x01(\x02\x12\x1f\n\x17maximum_review_interval\x18\x10 \x01(\r\x12\x1e\n\x16minimum_lapse_interval\x18\x11 \x01(\r\x12 \n\x18graduating_interval_good\x18\x12 \x01(\r\x12 \n\x18graduating_interval_easy\x18\x13 \x01(\r\x12U\n\x15new_card_insert_order\x18\x14 \x01(\x0e\x32\x36.anki.deck_config.DeckConfig.Config.NewCardInsertOrder\x12[\n\x18new_card_gather_priority\x18\" \x01(\x0e\x32\x39.anki.deck_config.DeckConfig.Config.NewCardGatherPriority\x12Q\n\x13new_card_sort_order\x18  \x01(\x0e\x32\x34.anki.deck_config.DeckConfig.Config.NewCardSortOrder\x12>\n\x07new_mix\x18\x1e \x01(\x0e\x32-.anki.deck_config.DeckConfig.Config.ReviewMix\x12I\n\x0creview_order\x18! \x01(\x0e\x32\x33.anki.deck_config.DeckConfig.Config.ReviewCardOrder\x12L\n\x15interday_learning_mix\x18\x1f \x01(\x0e\x32-.anki.deck_config.DeckConfig.Config.ReviewMix\x12\x45\n\x0cleech_action\x18\x15 \x01(\x0e\x32/.anki.deck_config.DeckConfig.Config.LeechAction\x12\x17\n\x0fleech_threshold\x18\x16 \x01(\r\x12\x18\n\x10\x64isable_autoplay\x18\x17 \x01(\x08\x12\x1f\n\x17\x63\x61p_answer_time_to_secs\x18\x18 \x01(\r\x12\x12\n\nshow_timer\x18\x19 \x01(\x08\x12\x1c\n\x14stop_timer_on_answer\x18& \x01(\x08\x12 \n\x18seconds_to_show_question\x18) \x01(\x02\x12\x1e\n\x16seconds_to_show_answer\x18* \x01(\x02\x12K\n\x0fquestion_action\x18$ \x01(\x0e\x32\x32.anki.deck_config.DeckConfig.Config.QuestionAction\x12G\n\ranswer_action\x18+ \x01(\x0e\x32\x30.anki.deck_config.DeckConfig.Config.AnswerAction\x12\x16\n\x0ewait_for_audio\x18, \x01(\x08\x12+\n#skip_question_when_replaying_answer\x18\x1a \x01(\x08\x12\x10\n\x08\x62ury_new\x18\x1b \x01(\x08\x12\x14\n\x0c\x62ury_reviews\x18\x1c \x01(\x08\x12\x1e\n\x16\x62ury_interday_learning\x18\x1d \x01(\x08\x12\x19\n\x11\x64\x65sired_retention\x18% \x01(\x02\x12\"\n\x1aignore_revlogs_before_date\x18. \x01(\t\x12\x1d\n\x15\x65\x61sy_days_percentages\x18\x04 \x03(\x02\x12\x1c\n\x14historical_retention\x18( \x01(\x02\x12\x14\n\x0cparam_search\x18- \x01(\t\x12\x0e\n\x05other\x18\xff\x01 \x01(\x0c\"U\n\x12NewCardInsertOrder\x12\x1d\n\x19NEW_CARD_INSERT_ORDER_DUE\x10\x00\x12 \n\x1cNEW_CARD_INSERT_ORDER_RANDOM\x10\x01\"\xa2\x02\n\x15NewCardGatherPriority\x12!\n\x1dNEW_CARD_GATHER_PRIORITY_DECK\x10\x00\x12\x33\n/NEW_CARD_GATHER_PRIORITY_DECK_THEN_RANDOM_NOTES\x10\x05\x12,\n(NEW_CARD_GATHER_PRIORITY_LOWEST_POSITION\x10\x01\x12-\n)NEW_CARD_GATHER_PRIORITY_HIGHEST_POSITION\x10\x02\x12)\n%NEW_CARD_GATHER_PRIORITY_RANDOM_NOTES\x10\x03\x12)\n%NEW_CARD_GATHER_PRIORITY_RANDOM_CARDS\x10\x04\"\xdb\x01\n\x10NewCardSortOrder\x12 \n\x1cNEW_CARD_SORT_ORDER_TEMPLATE\x10\x00\x12\x1f\n\x1bNEW_CARD_SORT_ORDER_NO_SORT\x10\x01\x12,\n(NEW_CARD_SORT_ORDER_TEMPLATE_THEN_RANDOM\x10\x02\x12\x31\n-NEW_CARD_SORT_ORDER_RANDOM_NOTE_THEN_TEMPLATE\x10\x03\x12#\n\x1fNEW_CARD_SORT_ORDER_RANDOM_CARD\x10\x04\"\xdb\x03\n\x0fReviewCardOrder\x12\x19\n\x15REVIEW_CARD_ORDER_DAY\x10\x00\x12#\n\x1fREVIEW_CARD_ORDER_DAY_THEN_DECK\x10\x01\x12#\n\x1fREVIEW_CARD_ORDER_DECK_THEN_DAY\x10\x02\x12)\n%REVIEW_CARD_ORDER_INTERVALS_ASCENDING\x10\x03\x12*\n&REVIEW_CARD_ORDER_INTERVALS_DESCENDING\x10\x04\x12$\n REVIEW_CARD_ORDER_EASE_ASCENDING\x10\x05\x12%\n!REVIEW_CARD_ORDER_EASE_DESCENDING\x10\x06\x12.\n*REVIEW_CARD_ORDER_RETRIEVABILITY_ASCENDING\x10\x07\x12/\n+REVIEW_CARD_ORDER_RETRIEVABILITY_DESCENDING\x10\x0b\x12\x1c\n\x18REVIEW_CARD_ORDER_RANDOM\x10\x08\x12\x1b\n\x17REVIEW_CARD_ORDER_ADDED\x10\t\x12#\n\x1fREVIEW_CARD_ORDER_REVERSE_ADDED\x10\n\"i\n\tReviewMix\x12\x1f\n\x1bREVIEW_MIX_MIX_WITH_REVIEWS\x10\x00\x12\x1c\n\x18REVIEW_MIX_AFTER_REVIEWS\x10\x01\x12\x1d\n\x19REVIEW_MIX_BEFORE_REVIEWS\x10\x02\"B\n\x0bLeechAction\x12\x18\n\x14LEECH_ACTION_SUSPEND\x10\x00\x12\x19\n\x15LEECH_ACTION_TAG_ONLY\x10\x01\"\xaa\x01\n\x0c\x41nswerAction\x12\x1b\n\x17\x41NSWER_ACTION_BURY_CARD\x10\x00\x12\x1e\n\x1a\x41NSWER_ACTION_ANSWER_AGAIN\x10\x01\x12\x1d\n\x19\x41NSWER_ACTION_ANSWER_GOOD\x10\x02\x12\x1d\n\x19\x41NSWER_ACTION_ANSWER_HARD\x10\x03\x12\x1f\n\x1b\x41NSWER_ACTION_SHOW_REMINDER\x10\x04\"T\n\x0eQuestionAction\x12\x1f\n\x1bQUESTION_ACTION_SHOW_ANSWER\x10\x00\x12!\n\x1dQUESTION_ACTION_SHOW_REMINDER\x10\x01J\x04\x08\x07\x10\tJ\x04\x08\'\x10(\"\xb9\x07\n\x14\x44\x65\x63kConfigsForUpdate\x12J\n\nall_config\x18\x01 \x03(\x0b\x32\x36.anki.deck_config.DeckConfigsForUpdate.ConfigWithExtra\x12H\n\x0c\x63urrent_deck\x18\x02 \x01(\x0b\x32\x32.anki.deck_config.DeckConfigsForUpdate.CurrentDeck\x12.\n\x08\x64\x65\x66\x61ults\x18\x03 \x01(\x0b\x32\x1c.anki.deck_config.DeckConfig\x12\x17\n\x0fschema_modified\x18\x04 \x01(\x08\x12\x1d\n\x15\x63\x61rd_state_customizer\x18\x06 \x01(\t\x12%\n\x1dnew_cards_ignore_review_limit\x18\x07 \x01(\x08\x12\x0c\n\x04\x66srs\x18\x08 \x01(\x08\x12\x19\n\x11\x66srs_health_check\x18\x0b \x01(\x08\x12\x1c\n\x14\x66srs_legacy_evaluate\x18\x0c \x01(\x08\x12\x1f\n\x17\x61pply_all_parent_limits\x18\t \x01(\x08\x12%\n\x1d\x64\x61ys_since_last_fsrs_optimize\x18\n \x01(\r\x1aR\n\x0f\x43onfigWithExtra\x12,\n\x06\x63onfig\x18\x01 \x01(\x0b\x32\x1c.anki.deck_config.DeckConfig\x12\x11\n\tuse_count\x18\x02 \x01(\r\x1a\x98\x03\n\x0b\x43urrentDeck\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\tconfig_id\x18\x02 \x01(\x03\x12\x19\n\x11parent_config_ids\x18\x03 \x03(\x03\x12I\n\x06limits\x18\x04 \x01(\x0b\x32\x39.anki.deck_config.DeckConfigsForUpdate.CurrentDeck.Limits\x1a\x81\x02\n\x06Limits\x12\x13\n\x06review\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x10\n\x03new\x18\x02 \x01(\rH\x01\x88\x01\x01\x12\x19\n\x0creview_today\x18\x03 \x01(\rH\x02\x88\x01\x01\x12\x16\n\tnew_today\x18\x04 \x01(\rH\x03\x88\x01\x01\x12\x1b\n\x13review_today_active\x18\x05 \x01(\x08\x12\x18\n\x10new_today_active\x18\x06 \x01(\x08\x12\x1e\n\x11\x64\x65sired_retention\x18\x07 \x01(\x02H\x04\x88\x01\x01\x42\t\n\x07_reviewB\x06\n\x04_newB\x0f\n\r_review_todayB\x0c\n\n_new_todayB\x14\n\x12_desired_retention\"\xa8\x03\n\x18UpdateDeckConfigsRequest\x12\x16\n\x0etarget_deck_id\x18\x01 \x01(\x03\x12-\n\x07\x63onfigs\x18\x02 \x03(\x0b\x32\x1c.anki.deck_config.DeckConfig\x12\x1a\n\x12removed_config_ids\x18\x03 \x03(\x03\x12\x35\n\x04mode\x18\x04 \x01(\x0e\x32\'.anki.deck_config.UpdateDeckConfigsMode\x12\x1d\n\x15\x63\x61rd_state_customizer\x18\x05 \x01(\t\x12I\n\x06limits\x18\x06 \x01(\x0b\x32\x39.anki.deck_config.DeckConfigsForUpdate.CurrentDeck.Limits\x12%\n\x1dnew_cards_ignore_review_limit\x18\x07 \x01(\x08\x12\x0c\n\x04\x66srs\x18\x08 \x01(\x08\x12\x1f\n\x17\x61pply_all_parent_limits\x18\t \x01(\x08\x12\x17\n\x0f\x66srs_reschedule\x18\n \x01(\x08\x12\x19\n\x11\x66srs_health_check\x18\x0b \x01(\x08*\x9d\x01\n\x15UpdateDeckConfigsMode\x12#\n\x1fUPDATE_DECK_CONFIGS_MODE_NORMAL\x10\x00\x12.\n*UPDATE_DECK_CONFIGS_MODE_APPLY_TO_CHILDREN\x10\x01\x12/\n+UPDATE_DECK_CONFIGS_MODE_COMPUTE_ALL_PARAMS\x10\x02\x32\xee\x06\n\x11\x44\x65\x63kConfigService\x12Q\n\x1b\x41\x64\x64OrUpdateDeckConfigLegacy\x12\x12.anki.generic.Json\x1a\x1e.anki.deck_config.DeckConfigId\x12M\n\rGetDeckConfig\x12\x1e.anki.deck_config.DeckConfigId\x1a\x1c.anki.deck_config.DeckConfig\x12>\n\x13\x41llDeckConfigLegacy\x12\x13.anki.generic.Empty\x1a\x12.anki.generic.Json\x12I\n\x13GetDeckConfigLegacy\x12\x1e.anki.deck_config.DeckConfigId\x1a\x12.anki.generic.Json\x12>\n\x13NewDeckConfigLegacy\x12\x13.anki.generic.Empty\x1a\x12.anki.generic.Json\x12G\n\x10RemoveDeckConfig\x12\x1e.anki.deck_config.DeckConfigId\x1a\x13.anki.generic.Empty\x12U\n\x17GetDeckConfigsForUpdate\x12\x12.anki.decks.DeckId\x1a&.anki.deck_config.DeckConfigsForUpdate\x12[\n\x11UpdateDeckConfigs\x12*.anki.deck_config.UpdateDeckConfigsRequest\x1a\x1a.anki.collection.OpChanges\x12x\n\x15GetIgnoredBeforeCount\x12..anki.deck_config.GetIgnoredBeforeCountRequest\x1a/.anki.deck_config.GetIgnoredBeforeCountResponse\x12u\n\x14GetRetentionWorkload\x12-.anki.deck_config.GetRetentionWorkloadRequest\x1a..anki.deck_config.GetRetentionWorkloadResponse2\x1a\n\x18\x42\x61\x63kendDeckConfigServiceB\x0c\x42\x08\x44\x65\x63kConfP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'anki.deck_config_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'B\010DeckConfP\001'
  _globals['_GETRETENTIONWORKLOADRESPONSE_COSTSENTRY']._loaded_options = None
  _globals['_GETRETENTIONWORKLOADRESPONSE_COSTSENTRY']._serialized_options = b'8\001'
  _globals['_UPDATEDECKCONFIGSMODE']._serialized_start=5196
  _globals['_UPDATEDECKCONFIGSMODE']._serialized_end=5353
  _globals['_DECKCONFIGID']._serialized_start=105
  _globals['_DECKCONFIGID']._serialized_end=133
  _globals['_GETRETENTIONWORKLOADREQUEST']._serialized_start=135
  _globals['_GETRETENTIONWORKLOADREQUEST']._serialized_end=191
  _globals['_GETRETENTIONWORKLOADRESPONSE']._serialized_start=194
  _globals['_GETRETENTIONWORKLOADRESPONSE']._serialized_end=344
  _globals['_GETRETENTIONWORKLOADRESPONSE_COSTSENTRY']._serialized_start=300
  _globals['_GETRETENTIONWORKLOADRESPONSE_COSTSENTRY']._serialized_end=344
  _globals['_GETIGNOREDBEFORECOUNTREQUEST']._serialized_start=346
  _globals['_GETIGNOREDBEFORECOUNTREQUEST']._serialized_end=428
  _globals['_GETIGNOREDBEFORECOUNTRESPONSE']._serialized_start=430
  _globals['_GETIGNOREDBEFORECOUNTRESPONSE']._serialized_end=494
  _globals['_DECKCONFIG']._serialized_start=497
  _globals['_DECKCONFIG']._serialized_end=3810
  _globals['_DECKCONFIG_CONFIG']._serialized_start=624
  _globals['_DECKCONFIG_CONFIG']._serialized_end=3810
  _globals['_DECKCONFIG_CONFIG_NEWCARDINSERTORDER']._serialized_start=2286
  _globals['_DECKCONFIG_CONFIG_NEWCARDINSERTORDER']._serialized_end=2371
  _globals['_DECKCONFIG_CONFIG_NEWCARDGATHERPRIORITY']._serialized_start=2374
  _globals['_DECKCONFIG_CONFIG_NEWCARDGATHERPRIORITY']._serialized_end=2664
  _globals['_DECKCONFIG_CONFIG_NEWCARDSORTORDER']._serialized_start=2667
  _globals['_DECKCONFIG_CONFIG_NEWCARDSORTORDER']._serialized_end=2886
  _globals['_DECKCONFIG_CONFIG_REVIEWCARDORDER']._serialized_start=2889
  _globals['_DECKCONFIG_CONFIG_REVIEWCARDORDER']._serialized_end=3364
  _globals['_DECKCONFIG_CONFIG_REVIEWMIX']._serialized_start=3366
  _globals['_DECKCONFIG_CONFIG_REVIEWMIX']._serialized_end=3471
  _globals['_DECKCONFIG_CONFIG_LEECHACTION']._serialized_start=3473
  _globals['_DECKCONFIG_CONFIG_LEECHACTION']._serialized_end=3539
  _globals['_DECKCONFIG_CONFIG_ANSWERACTION']._serialized_start=3542
  _globals['_DECKCONFIG_CONFIG_ANSWERACTION']._serialized_end=3712
  _globals['_DECKCONFIG_CONFIG_QUESTIONACTION']._serialized_start=3714
  _globals['_DECKCONFIG_CONFIG_QUESTIONACTION']._serialized_end=3798
  _globals['_DECKCONFIGSFORUPDATE']._serialized_start=3813
  _globals['_DECKCONFIGSFORUPDATE']._serialized_end=4766
  _globals['_DECKCONFIGSFORUPDATE_CONFIGWITHEXTRA']._serialized_start=4273
  _globals['_DECKCONFIGSFORUPDATE_CONFIGWITHEXTRA']._serialized_end=4355
  _globals['_DECKCONFIGSFORUPDATE_CURRENTDECK']._serialized_start=4358
  _globals['_DECKCONFIGSFORUPDATE_CURRENTDECK']._serialized_end=4766
  _globals['_DECKCONFIGSFORUPDATE_CURRENTDECK_LIMITS']._serialized_start=4509
  _globals['_DECKCONFIGSFORUPDATE_CURRENTDECK_LIMITS']._serialized_end=4766
  _globals['_UPDATEDECKCONFIGSREQUEST']._serialized_start=4769
  _globals['_UPDATEDECKCONFIGSREQUEST']._serialized_end=5193
  _globals['_DECKCONFIGSERVICE']._serialized_start=5356
  _globals['_DECKCONFIGSERVICE']._serialized_end=6234
  _globals['_BACKENDDECKCONFIGSERVICE']._serialized_start=6236
  _globals['_BACKENDDECKCONFIGSERVICE']._serialized_end=6262
# @@protoc_insertion_point(module_scope)



# Form implementation generated from reading ui file 'qt/aqt/forms/dconf.ui'
#
# Created by: PyQt6 UI code generator 6.11.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets
from aqt.utils import tr



class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(638, 514)
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_31 = QtWidgets.QLabel(parent=Dialog)
        self.label_31.setObjectName("label_31")
        self.horizontalLayout_2.addWidget(self.label_31)
        self.dconf = QtWidgets.QComboBox(parent=Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(3)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.dconf.sizePolicy().hasHeightForWidth())
        self.dconf.setSizePolicy(sizePolicy)
        self.dconf.setObjectName("dconf")
        self.horizontalLayout_2.addWidget(self.dconf)
        self.confOpts = QtWidgets.QToolButton(parent=Dialog)
        self.confOpts.setMaximumSize(QtCore.QSize(16777215, 32))
        self.confOpts.setToolButtonStyle(QtCore.Qt.ToolButtonStyle.ToolButtonTextBesideIcon)
        self.confOpts.setArrowType(QtCore.Qt.ArrowType.NoArrow)
        self.confOpts.setObjectName("confOpts")
        self.horizontalLayout_2.addWidget(self.confOpts)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.count = QtWidgets.QLabel(parent=Dialog)
        self.count.setStyleSheet("* { color: red }")
        self.count.setText("")
        self.count.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.count.setWordWrap(True)
        self.count.setObjectName("count")
        self.verticalLayout.addWidget(self.count)
        self.tabWidget = QtWidgets.QTabWidget(parent=Dialog)
        self.tabWidget.setObjectName("tabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.tab)
        self.verticalLayout_2.setContentsMargins(12, 12, 12, 12)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setSpacing(12)
        self.gridLayout.setObjectName("gridLayout")
        self.label_24 = QtWidgets.QLabel(parent=self.tab)
        self.label_24.setObjectName("label_24")
        self.gridLayout.addWidget(self.label_24, 5, 0, 1, 1)
        self.lrnFactor = QtWidgets.QSpinBox(parent=self.tab)
        self.lrnFactor.setSuffix("%")
        self.lrnFactor.setMinimum(131)
        self.lrnFactor.setMaximum(999)
        self.lrnFactor.setProperty("value", 250)
        self.lrnFactor.setObjectName("lrnFactor")
        self.gridLayout.addWidget(self.lrnFactor, 5, 1, 1, 1)
        self.label_8 = QtWidgets.QLabel(parent=self.tab)
        self.label_8.setObjectName("label_8")
        self.gridLayout.addWidget(self.label_8, 1, 0, 1, 1)
        self.lrnEasyInt = QtWidgets.QSpinBox(parent=self.tab)
        self.lrnEasyInt.setMinimum(1)
        self.lrnEasyInt.setObjectName("lrnEasyInt")
        self.gridLayout.addWidget(self.lrnEasyInt, 4, 1, 1, 1)
        self.lrnGradInt = QtWidgets.QSpinBox(parent=self.tab)
        self.lrnGradInt.setMinimum(1)
        self.lrnGradInt.setObjectName("lrnGradInt")
        self.gridLayout.addWidget(self.lrnGradInt, 3, 1, 1, 1)
        self.newplim = QtWidgets.QLabel(parent=self.tab)
        self.newplim.setText("")
        self.newplim.setObjectName("newplim")
        self.gridLayout.addWidget(self.newplim, 2, 2, 1, 1)
        self.label_5 = QtWidgets.QLabel(parent=self.tab)
        self.label_5.setObjectName("label_5")
        self.gridLayout.addWidget(self.label_5, 4, 0, 1, 1)
        self.label_4 = QtWidgets.QLabel(parent=self.tab)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 3, 0, 1, 1)
        self.newPerDay = QtWidgets.QSpinBox(parent=self.tab)
        self.newPerDay.setMaximum(9999)
        self.newPerDay.setObjectName("newPerDay")
        self.gridLayout.addWidget(self.newPerDay, 2, 1, 1, 1)
        self.label_6 = QtWidgets.QLabel(parent=self.tab)
        self.label_6.setObjectName("label_6")
        self.gridLayout.addWidget(self.label_6, 2, 0, 1, 1)
        self.lrnSteps = QtWidgets.QLineEdit(parent=self.tab)
        self.lrnSteps.setObjectName("lrnSteps")
        self.gridLayout.addWidget(self.lrnSteps, 0, 1, 1, 2)
        self.label_2 = QtWidgets.QLabel(parent=self.tab)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 0, 0, 1, 1)
        self.newOrder = QtWidgets.QComboBox(parent=self.tab)
        self.newOrder.setObjectName("newOrder")
        self.gridLayout.addWidget(self.newOrder, 1, 1, 1, 2)
        self.bury = QtWidgets.QCheckBox(parent=self.tab)
        self.bury.setObjectName("bury")
        self.gridLayout.addWidget(self.bury, 6, 0, 1, 3, QtCore.Qt.AlignmentFlag.AlignLeft)
        self.label_9 = QtWidgets.QLabel(parent=self.tab)
        self.label_9.setObjectName("label_9")
        self.gridLayout.addWidget(self.label_9, 4, 2, 1, 1)
        self.label_7 = QtWidgets.QLabel(parent=self.tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_7.sizePolicy().hasHeightForWidth())
        self.label_7.setSizePolicy(sizePolicy)
        self.label_7.setObjectName("label_7")
        self.gridLayout.addWidget(self.label_7, 3, 2, 1, 1)
        self.verticalLayout_2.addLayout(self.gridLayout)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout_2.addItem(spacerItem)
        self.tabWidget.addTab(self.tab, "")
        self.tab_3 = QtWidgets.QWidget()
        self.tab_3.setObjectName("tab_3")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.tab_3)
        self.verticalLayout_4.setContentsMargins(12, 12, 12, 12)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setSpacing(12)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.label_20 = QtWidgets.QLabel(parent=self.tab_3)
        self.label_20.setObjectName("label_20")
        self.gridLayout_3.addWidget(self.label_20, 1, 0, 1, 1)
        self.easyBonus = QtWidgets.QSpinBox(parent=self.tab_3)
        self.easyBonus.setSuffix("%")
        self.easyBonus.setMinimum(100)
        self.easyBonus.setMaximum(1000)
        self.easyBonus.setSingleStep(5)
        self.easyBonus.setObjectName("easyBonus")
        self.gridLayout_3.addWidget(self.easyBonus, 1, 1, 1, 1)
        self.revPerDay = QtWidgets.QSpinBox(parent=self.tab_3)
        self.revPerDay.setMinimum(0)
        self.revPerDay.setMaximum(9999)
        self.revPerDay.setObjectName("revPerDay")
        self.gridLayout_3.addWidget(self.revPerDay, 0, 1, 1, 1)
        self.label_33 = QtWidgets.QLabel(parent=self.tab_3)
        self.label_33.setObjectName("label_33")
        self.gridLayout_3.addWidget(self.label_33, 2, 0, 1, 1)
        self.label_37 = QtWidgets.QLabel(parent=self.tab_3)
        self.label_37.setObjectName("label_37")
        self.gridLayout_3.addWidget(self.label_37, 0, 0, 1, 1)
        self.label_3 = QtWidgets.QLabel(parent=self.tab_3)
        self.label_3.setObjectName("label_3")
        self.gridLayout_3.addWidget(self.label_3, 3, 0, 1, 1)
        self.maxIvl = QtWidgets.QSpinBox(parent=self.tab_3)
        self.maxIvl.setSuffix("")
        self.maxIvl.setMinimum(1)
        self.maxIvl.setMaximum(99999)
        self.maxIvl.setObjectName("maxIvl")
        self.gridLayout_3.addWidget(self.maxIvl, 3, 1, 1, 1)
        self.label_23 = QtWidgets.QLabel(parent=self.tab_3)
        self.label_23.setObjectName("label_23")
        self.gridLayout_3.addWidget(self.label_23, 3, 2, 1, 1)
        self.revplim = QtWidgets.QLabel(parent=self.tab_3)
        self.revplim.setText("")
        self.revplim.setObjectName("revplim")
        self.gridLayout_3.addWidget(self.revplim, 0, 2, 1, 1)
        self.fi1 = QtWidgets.QDoubleSpinBox(parent=self.tab_3)
        self.fi1.setSuffix("%")
        self.fi1.setDecimals(0)
        self.fi1.setMinimum(0.0)
        self.fi1.setMaximum(999.0)
        self.fi1.setSingleStep(1.0)
        self.fi1.setProperty("value", 100.0)
        self.fi1.setObjectName("fi1")
        self.gridLayout_3.addWidget(self.fi1, 2, 1, 1, 1)
        self.buryRev = QtWidgets.QCheckBox(parent=self.tab_3)
        self.buryRev.setObjectName("buryRev")
        self.gridLayout_3.addWidget(self.buryRev, 5, 0, 1, 3, QtCore.Qt.AlignmentFlag.AlignLeft)
        self.hardFactorLabel = QtWidgets.QLabel(parent=self.tab_3)
        self.hardFactorLabel.setObjectName("hardFactorLabel")
        self.gridLayout_3.addWidget(self.hardFactorLabel, 4, 0, 1, 1)
        self.hardFactor = QtWidgets.QSpinBox(parent=self.tab_3)
        self.hardFactor.setSuffix("%")
        self.hardFactor.setMinimum(5)
        self.hardFactor.setMaximum(120)
        self.hardFactor.setObjectName("hardFactor")
        self.gridLayout_3.addWidget(self.hardFactor, 4, 1, 1, 1)
        self.verticalLayout_4.addLayout(self.gridLayout_3)
        spacerItem1 = QtWidgets.QSpacerItem(20, 152, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout_4.addItem(spacerItem1)
        self.tabWidget.addTab(self.tab_3, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.tab_2)
        self.verticalLayout_3.setContentsMargins(12, 12, 12, 12)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setSpacing(12)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.label_17 = QtWidgets.QLabel(parent=self.tab_2)
        self.label_17.setObjectName("label_17")
        self.gridLayout_2.addWidget(self.label_17, 0, 0, 1, 1)
        self.lapSteps = QtWidgets.QLineEdit(parent=self.tab_2)
        self.lapSteps.setObjectName("lapSteps")
        self.gridLayout_2.addWidget(self.lapSteps, 0, 1, 1, 2)
        self.label = QtWidgets.QLabel(parent=self.tab_2)
        self.label.setObjectName("label")
        self.gridLayout_2.addWidget(self.label, 1, 0, 1, 1)
        self.label_10 = QtWidgets.QLabel(parent=self.tab_2)
        self.label_10.setObjectName("label_10")
        self.gridLayout_2.addWidget(self.label_10, 3, 0, 1, 1)
        self.leechThreshold = QtWidgets.QSpinBox(parent=self.tab_2)
        self.leechThreshold.setObjectName("leechThreshold")
        self.gridLayout_2.addWidget(self.leechThreshold, 3, 1, 1, 1)
        self.label_11 = QtWidgets.QLabel(parent=self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_11.sizePolicy().hasHeightForWidth())
        self.label_11.setSizePolicy(sizePolicy)
        self.label_11.setObjectName("label_11")
        self.gridLayout_2.addWidget(self.label_11, 3, 2, 1, 1)
        self.label_12 = QtWidgets.QLabel(parent=self.tab_2)
        self.label_12.setObjectName("label_12")
        self.gridLayout_2.addWidget(self.label_12, 4, 0, 1, 1)
        self.lapMinInt = QtWidgets.QSpinBox(parent=self.tab_2)
        self.lapMinInt.setMinimum(1)
        self.lapMinInt.setMaximum(99)
        self.lapMinInt.setObjectName("lapMinInt")
        self.gridLayout_2.addWidget(self.lapMinInt, 2, 1, 1, 1)
        self.label_13 = QtWidgets.QLabel(parent=self.tab_2)
        self.label_13.setObjectName("label_13")
        self.gridLayout_2.addWidget(self.label_13, 2, 0, 1, 1)
        self.label_14 = QtWidgets.QLabel(parent=self.tab_2)
        self.label_14.setObjectName("label_14")
        self.gridLayout_2.addWidget(self.label_14, 2, 2, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.leechAction = QtWidgets.QComboBox(parent=self.tab_2)
        self.leechAction.setObjectName("leechAction")
        self.leechAction.addItem("")
        self.leechAction.addItem("")
        self.horizontalLayout.addWidget(self.leechAction)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout.addItem(spacerItem2)
        self.gridLayout_2.addLayout(self.horizontalLayout, 4, 1, 1, 2)
        self.lapMult = QtWidgets.QSpinBox(parent=self.tab_2)
        self.lapMult.setSuffix("%")
        self.lapMult.setMaximum(100)
        self.lapMult.setSingleStep(5)
        self.lapMult.setObjectName("lapMult")
        self.gridLayout_2.addWidget(self.lapMult, 1, 1, 1, 1)
        self.verticalLayout_3.addLayout(self.gridLayout_2)
        spacerItem3 = QtWidgets.QSpacerItem(20, 72, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout_3.addItem(spacerItem3)
        self.tabWidget.addTab(self.tab_2, "")
        self.tab_5 = QtWidgets.QWidget()
        self.tab_5.setObjectName("tab_5")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.tab_5)
        self.verticalLayout_6.setContentsMargins(12, 12, 12, 12)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.gridLayout_5 = QtWidgets.QGridLayout()
        self.gridLayout_5.setSpacing(12)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.label_25 = QtWidgets.QLabel(parent=self.tab_5)
        self.label_25.setObjectName("label_25")
        self.gridLayout_5.addWidget(self.label_25, 0, 0, 1, 1)
        self.maxTaken = QtWidgets.QSpinBox(parent=self.tab_5)
        self.maxTaken.setMinimum(30)
        self.maxTaken.setMaximum(3600)
        self.maxTaken.setSingleStep(10)
        self.maxTaken.setObjectName("maxTaken")
        self.gridLayout_5.addWidget(self.maxTaken, 0, 1, 1, 1)
        self.label_26 = QtWidgets.QLabel(parent=self.tab_5)
        self.label_26.setObjectName("label_26")
        self.gridLayout_5.addWidget(self.label_26, 0, 2, 1, 1)
        self.verticalLayout_6.addLayout(self.gridLayout_5)
        self.showTimer = QtWidgets.QCheckBox(parent=self.tab_5)
        self.showTimer.setObjectName("showTimer")
        self.verticalLayout_6.addWidget(self.showTimer, 0, QtCore.Qt.AlignmentFlag.AlignLeft)
        self.autoplaySounds = QtWidgets.QCheckBox(parent=self.tab_5)
        self.autoplaySounds.setObjectName("autoplaySounds")
        self.verticalLayout_6.addWidget(self.autoplaySounds, 0, QtCore.Qt.AlignmentFlag.AlignLeft)
        self.replayQuestion = QtWidgets.QCheckBox(parent=self.tab_5)
        self.replayQuestion.setChecked(False)
        self.replayQuestion.setObjectName("replayQuestion")
        self.verticalLayout_6.addWidget(self.replayQuestion, 0, QtCore.Qt.AlignmentFlag.AlignLeft)
        spacerItem4 = QtWidgets.QSpacerItem(20, 199, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout_6.addItem(spacerItem4)
        self.tabWidget.addTab(self.tab_5, "")
        self.verticalLayout.addWidget(self.tabWidget)
        self.buttonBox = QtWidgets.QDialogButtonBox(parent=Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Help|QtWidgets.QDialogButtonBox.StandardButton.Ok|QtWidgets.QDialogButtonBox.StandardButton.RestoreDefaults)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(Dialog)
        self.tabWidget.setCurrentIndex(0)
        self.buttonBox.accepted.connect(Dialog.accept) # type: ignore  # type: ignore
        self.buttonBox.rejected.connect(Dialog.reject) # type: ignore  # type: ignore
        QtCore.QMetaObject.connectSlotsByName(Dialog)
        Dialog.setTabOrder(self.dconf, self.confOpts)
        Dialog.setTabOrder(self.confOpts, self.tabWidget)
        Dialog.setTabOrder(self.tabWidget, self.lrnSteps)
        Dialog.setTabOrder(self.lrnSteps, self.newOrder)
        Dialog.setTabOrder(self.newOrder, self.newPerDay)
        Dialog.setTabOrder(self.newPerDay, self.lrnGradInt)
        Dialog.setTabOrder(self.lrnGradInt, self.lrnEasyInt)
        Dialog.setTabOrder(self.lrnEasyInt, self.lrnFactor)
        Dialog.setTabOrder(self.lrnFactor, self.bury)
        Dialog.setTabOrder(self.bury, self.revPerDay)
        Dialog.setTabOrder(self.revPerDay, self.easyBonus)
        Dialog.setTabOrder(self.easyBonus, self.fi1)
        Dialog.setTabOrder(self.fi1, self.maxIvl)
        Dialog.setTabOrder(self.maxIvl, self.hardFactor)
        Dialog.setTabOrder(self.hardFactor, self.buryRev)
        Dialog.setTabOrder(self.buryRev, self.lapSteps)
        Dialog.setTabOrder(self.lapSteps, self.lapMult)
        Dialog.setTabOrder(self.lapMult, self.lapMinInt)
        Dialog.setTabOrder(self.lapMinInt, self.leechThreshold)
        Dialog.setTabOrder(self.leechThreshold, self.leechAction)
        Dialog.setTabOrder(self.leechAction, self.maxTaken)
        Dialog.setTabOrder(self.maxTaken, self.showTimer)
        Dialog.setTabOrder(self.showTimer, self.autoplaySounds)
        Dialog.setTabOrder(self.autoplaySounds, self.replayQuestion)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        self.label_31.setText(tr.scheduling_options_group())
        self.confOpts.setText(tr.actions_manage())
        self.label_24.setText(tr.scheduling_starting_ease())
        self.label_8.setText(tr.scheduling_order())
        self.label_5.setText(tr.scheduling_easy_interval())
        self.label_4.setText(tr.scheduling_graduating_interval())
        self.label_6.setText(tr.scheduling_new_cardsday())
        self.label_2.setText(tr.scheduling_steps_in_minutes())
        self.bury.setText(tr.scheduling_bury_related_new_cards_until_the())
        self.label_9.setText(tr.scheduling_days())
        self.label_7.setText(tr.scheduling_days())
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), tr.scheduling_new_cards())
        self.label_20.setText(tr.scheduling_easy_bonus())
        self.label_33.setText(tr.scheduling_interval_modifier())
        self.label_37.setText(tr.scheduling_maximum_reviewsday())
        self.label_3.setText(tr.scheduling_maximum_interval())
        self.label_23.setText(tr.scheduling_days())
        self.buryRev.setText(tr.scheduling_bury_related_reviews_until_the_next())
        self.hardFactorLabel.setText(tr.scheduling_hard_interval())
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_3), tr.scheduling_reviews())
        self.label_17.setText(tr.scheduling_steps_in_minutes())
        self.label.setText(tr.scheduling_new_interval())
        self.label_10.setText(tr.scheduling_leech_threshold())
        self.label_11.setText(tr.scheduling_lapses2())
        self.label_12.setText(tr.scheduling_leech_action())
        self.label_13.setText(tr.scheduling_minimum_interval())
        self.label_14.setText(tr.scheduling_days())
        self.leechAction.setItemText(0, tr.actions_suspend_card())
        self.leechAction.setItemText(1, tr.scheduling_tag_only())
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_2), tr.scheduling_lapses())
        self.label_25.setText(tr.scheduling_ignore_answer_times_longer_than())
        self.label_26.setText(tr.scheduling_seconds())
        self.showTimer.setText(tr.scheduling_show_answer_timer())
        self.autoplaySounds.setText(tr.scheduling_automatically_play_audio())
        self.replayQuestion.setText(tr.scheduling_always_include_question_side_when_replaying())
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_5), tr.scheduling_general())


# Copyright: Ankitects Pty Ltd and contributors
# License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html


import re
import time
from typing import cast

from anki.db import DB
from anki.importing.noteimp import ForeignCard, ForeignNote, NoteImporter
from anki.stdmodels import _legacy_add_basic_model, _legacy_add_cloze_model


class MnemosyneImporter(NoteImporter):
    needMapper = False
    update = False
    allowHTML = True

    def run(self):
        db = DB(self.file)
        ver = db.scalar("select value from global_variables where key='version'")
        if not ver.startswith("Mnemosyne SQL 1") and ver not in ("2", "3"):
            self.log.append(
                self.col.tr.importing_file_version_unknown_trying_import_anyway()
            )
        # gather facts into temp objects
        curid = None
        notes = {}
        note = None
        for _id, id, k, v in db.execute(
            """
select _id, id, key, value from facts f, data_for_fact d where
f._id=d._fact_id"""
        ):
            if id != curid:
                if note:
                    notes[note["_id"]] = note
                note = {"_id": _id}
                curid = id
            assert note
            note[k] = v
        if note:
            notes[note["_id"]] = note
        # gather cards
        front = []
        frontback = []
        vocabulary = []
        cloze = {}
        for row in db.execute(
            """
select _fact_id, fact_view_id, tags, next_rep, last_rep, easiness,
acq_reps+ret_reps, lapses, card_type_id from cards"""
        ):
            # categorize note
            note = notes[row[0]]
            if row[1].endswith(".1"):
                if row[1].startswith("1.") or row[1].startswith("1::"):
                    front.append(note)
                elif row[1].startswith("2.") or row[1].startswith("2::"):
                    frontback.append(note)
                elif row[1].startswith("3.") or row[1].startswith("3::"):
                    vocabulary.append(note)
                elif row[1].startswith("5.1"):
                    cloze[row[0]] = note
            # check for None to fix issue where import can error out
            rawTags = row[2]
            if rawTags is None:
                rawTags = ""
            # merge tags into note
            tags = rawTags.replace(", ", "\x1f").replace(" ", "_")
            tags = tags.replace("\x1f", " ")
            if "tags" not in note:
                note["tags"] = []
            note["tags"] += self.col.tags.split(tags)
            # if it's a new card we can go with the defaults
            if row[3] == -1:
                continue
            # add the card
            c = ForeignCard()
            c.factor = int(row[5] * 1000)
            c.reps = row[6]
            c.lapses = row[7]
            # ivl is inferred in mnemosyne
            next, prev = row[3:5]
            c.ivl = max(1, (next - prev) // 86400)
            # work out how long we've got left
            rem = int((next - time.time()) / 86400)
            c.due = self.col.sched.today + rem
            # get ord
            m = re.search(r".(\d+)$", row[1])
            assert m
            ord = int(m.group(1)) - 1
            if "cards" not in note:
                note["cards"] = {}
            note["cards"][ord] = c
        self._addFronts(front)
        total = self.total
        self._addFrontBacks(frontback)
        total += self.total
        self._addVocabulary(vocabulary)
        self.total += total
        self._addCloze(cloze)
        self.total += total
        self.log.append(self.col.tr.importing_note_imported(count=self.total))

    def fields(self):
        return self._fields

    def _mungeField(self, fld):
        # \n -> br
        fld = re.sub("\r?\n", "<br>", fld)
        # latex differences
        fld = re.sub(r"(?i)<(/?(\$|\$\$|latex))>", "[\\1]", fld)
        # audio differences
        fld = re.sub('<audio src="(.+?)">(</audio>)?', "[sound:\\1]", fld)
        return fld

    def _addFronts(self, notes, model=None, fields=("f", "b")):
        data = []
        for orig in notes:
            # create a foreign note object
            n = ForeignNote()
            n.fields = []
            for f in fields:
                fld = self._mungeField(orig.get(f, ""))
                n.fields.append(fld)
            n.tags = orig["tags"]
            n.cards = orig.get("cards", {})
            data.append(n)
        # add a basic model
        if not model:
            model = _legacy_add_basic_model(self.col)
            model["name"] = "Mnemosyne-FrontOnly"
        mm = self.col.models
        mm.save(model)
        mm.set_current(model)
        self.model = model
        self._fields = len(model["flds"])
        self.initMapping()
        # import
        self.importNotes(data)

    def _addFrontBacks(self, notes):
        m = _legacy_add_basic_model(self.col)
        m["name"] = "Mnemosyne-FrontBack"
        mm = self.col.models
        t = mm.new_template("Back")
        t["qfmt"] = "{{Back}}"
        t["afmt"] = f"{t['qfmt']}\n\n<hr id=answer>\n\n{{{{Front}}}}"  # type: ignore
        mm.add_template(m, t)
        self._addFronts(notes, m)

    def _addVocabulary(self, notes):
        mm = self.col.models
        m = mm.new("Mnemosyne-Vocabulary")
        for f in "Expression", "Pronunciation", "Meaning", "Notes":
            fm = mm.new_field(f)
            mm.addField(m, fm)
        t = mm.new_template("Recognition")
        t["qfmt"] = "{{Expression}}"
        t["afmt"] = (
            f"{cast(str, t['qfmt'])}\n\n<hr id=answer>\n\n{{{{Pronunciation}}}}<br>\n{{{{Meaning}}}}<br>\n{{{{Notes}}}}"
        )
        mm.add_template(m, t)
        t = mm.new_template("Production")
        t["qfmt"] = "{{Meaning}}"
        t["afmt"] = (
            f"{cast(str, t['qfmt'])}\n\n<hr id=answer>\n\n{{{{Expression}}}}<br>\n{{{{Pronunciation}}}}<br>\n{{{{Notes}}}}"
        )
        mm.add_template(m, t)
        mm.add(m)
        self._addFronts(notes, m, fields=("f", "p_1", "m_1", "n"))

    def _addCloze(self, notes):
        data = []
        notes = list(notes.values())
        for orig in notes:
            # create a foreign note object
            n = ForeignNote()
            n.fields = []
            fld = orig.get("text", "")
            fld = re.sub("\r?\n", "<br>", fld)
            state = dict(n=1)

            def repl(match):
                # replace [...] with cloze refs
                res = "{{c%d::%s}}" % (state["n"], match.group(1))
                state["n"] += 1
                return res

            fld = re.sub(r"\[(.+?)\]", repl, fld)
            fld = self._mungeField(fld)
            n.fields.append(fld)
            n.fields.append("")  # extra
            n.tags = orig["tags"]
            n.cards = orig.get("cards", {})
            data.append(n)
        # add cloze model
        model = _legacy_add_cloze_model(self.col)
        model["name"] = "Mnemosyne-Cloze"
        mm = self.col.models
        mm.save(model)
        mm.set_current(model)
        self.model = model
        self._fields = len(model["flds"])
        self.initMapping()
        self.importNotes(data)



# Form implementation generated from reading ui file 'qt/aqt/forms/studydeck.ui'
#
# Created by: PyQt6 UI code generator 6.11.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets
from aqt.utils import tr



class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(400, 300)
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(parent=Dialog)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.filter = QtWidgets.QLineEdit(parent=Dialog)
        self.filter.setObjectName("filter")
        self.horizontalLayout.addWidget(self.filter)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.list = QtWidgets.QListWidget(parent=Dialog)
        self.list.setObjectName("list")
        self.verticalLayout.addWidget(self.list)
        self.buttonBox = QtWidgets.QDialogButtonBox(parent=Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Cancel|QtWidgets.QDialogButtonBox.StandardButton.Help)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(Dialog)
        self.buttonBox.accepted.connect(Dialog.accept) # type: ignore  # type: ignore
        self.buttonBox.rejected.connect(Dialog.reject) # type: ignore  # type: ignore
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(tr.decks_study_deck())
        self.label.setText(tr.decks_filter())


# Copyright: Ankitects Pty Ltd and contributors
# License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html

"""
Wrapper for requests that adds a callback for tracking upload/download progress.
"""

from __future__ import annotations

import io
import os
from collections.abc import Callable
from typing import Any

import requests
from requests import Response

from anki._legacy import DeprecatedNamesMixin

HTTP_BUF_SIZE = 64 * 1024

ProgressCallback = Callable[[int, int], None]


class HttpClient(DeprecatedNamesMixin):
    verify = True
    timeout = 60
    # args are (upload_bytes_in_chunk, download_bytes_in_chunk)
    progress_hook: ProgressCallback | None = None

    def __init__(self, progress_hook: ProgressCallback | None = None) -> None:
        self.progress_hook = progress_hook
        self.session = requests.Session()

    def __enter__(self) -> HttpClient:
        return self

    def __exit__(self, *args: Any) -> None:
        self.close()

    def close(self) -> None:
        if self.session:
            self.session.close()
            self.session = None

    def __del__(self) -> None:
        self.close()

    def post(self, url: str, data: bytes, headers: dict[str, str] | None) -> Response:
        headers["User-Agent"] = self._agent_name()
        return self.session.post(
            url,
            data=data,
            headers=headers,
            stream=True,
            timeout=self.timeout,
            verify=self.verify,
        )  # pytype: disable=wrong-arg-types

    def get(self, url: str, headers: dict[str, str] | None = None) -> Response:
        if headers is None:
            headers = {}
        headers["User-Agent"] = self._agent_name()
        return self.session.get(
            url, stream=True, headers=headers, timeout=self.timeout, verify=self.verify
        )

    def stream_content(self, resp: Response) -> bytes:
        resp.raise_for_status()

        buf = io.BytesIO()
        for chunk in resp.iter_content(chunk_size=HTTP_BUF_SIZE):
            if self.progress_hook:
                self.progress_hook(0, len(chunk))
            buf.write(chunk)
        return buf.getvalue()

    def _agent_name(self) -> str:
        from anki.buildinfo import version

        return f"Anki {version}"


# allow user to accept invalid certs in work/school settings
if os.environ.get("ANKI_NOVERIFYSSL"):
    HttpClient.verify = False

    import warnings

    warnings.filterwarnings("ignore")



# Copyright: Ankitects Pty Ltd and contributors
# License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html


class BrowserConfig:
    ACTIVE_CARD_COLUMNS_KEY = "activeCols"
    ACTIVE_NOTE_COLUMNS_KEY = "activeNoteCols"
    CARDS_SORT_COLUMN_KEY = "sortType"
    NOTES_SORT_COLUMN_KEY = "noteSortType"
    CARDS_SORT_BACKWARDS_KEY = "sortBackwards"
    NOTES_SORT_BACKWARDS_KEY = "browserNoteSortBackwards"

    @staticmethod
    def active_columns_key(is_notes_mode: bool) -> str:
        if is_notes_mode:
            return BrowserConfig.ACTIVE_NOTE_COLUMNS_KEY
        return BrowserConfig.ACTIVE_CARD_COLUMNS_KEY

    @staticmethod
    def sort_column_key(is_notes_mode: bool) -> str:
        if is_notes_mode:
            return BrowserConfig.NOTES_SORT_COLUMN_KEY
        return BrowserConfig.CARDS_SORT_COLUMN_KEY

    @staticmethod
    def sort_backwards_key(is_notes_mode: bool) -> str:
        if is_notes_mode:
            return BrowserConfig.NOTES_SORT_BACKWARDS_KEY
        return BrowserConfig.CARDS_SORT_BACKWARDS_KEY


class BrowserDefaults:
    CARD_COLUMNS = ["noteFld", "template", "cardDue", "deck"]
    NOTE_COLUMNS = ["noteFld", "note", "template", "noteTags"]



# Form implementation generated from reading ui file 'qt/aqt/forms/debug.ui'
#
# Created by: PyQt6 UI code generator 6.11.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets
from aqt.utils import tr



class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(637, 582)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("icons:anki.png"), QtGui.QIcon.Mode.Normal, QtGui.QIcon.State.Off)
        Dialog.setWindowIcon(icon)
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.script = QtWidgets.QComboBox(parent=Dialog)
        self.script.setCurrentText("")
        self.script.setObjectName("script")
        self.verticalLayout.addWidget(self.script)
        self.splitter = QtWidgets.QSplitter(parent=Dialog)
        self.splitter.setOrientation(QtCore.Qt.Orientation.Vertical)
        self.splitter.setChildrenCollapsible(False)
        self.splitter.setObjectName("splitter")
        self.text = QtWidgets.QPlainTextEdit(parent=self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.text.sizePolicy().hasHeightForWidth())
        self.text.setSizePolicy(sizePolicy)
        self.text.setMinimumSize(QtCore.QSize(0, 100))
        self.text.setMaximumSize(QtCore.QSize(16777215, 1677215))
        self.text.setLineWrapMode(QtWidgets.QPlainTextEdit.LineWrapMode.NoWrap)
        self.text.setPlaceholderText("Actions:\n"
"    Ctrl+Enter          Execute\n"
"    Ctrl+Shift+Enter    Execute and print current line\n"
"    Ctrl+L              Clear log\n"
"    Ctrl+Shift+L        Clear input\n"
"    Ctrl+S              Save script\n"
"    Ctrl+O              Open script\n"
"    Ctrl+D              Delete script\n"
"\n"
"Locals:\n"
"    mw: AnkiQt                          Main window\n"
"    card: Callable[[], Card | None]     Reviewer card\n"
"    bcard: Callable[[], Card | None]    Browser card\n"
"    pp: Callable[[object], None]        Pretty print")
        self.text.setObjectName("text")
        self.log = QtWidgets.QPlainTextEdit(parent=self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(8)
        sizePolicy.setHeightForWidth(self.log.sizePolicy().hasHeightForWidth())
        self.log.setSizePolicy(sizePolicy)
        self.log.setMinimumSize(QtCore.QSize(0, 150))
        self.log.setFocusPolicy(QtCore.Qt.FocusPolicy.ClickFocus)
        self.log.setReadOnly(True)
        self.log.setPlaceholderText("Output")
        self.log.setObjectName("log")
        self.verticalLayout.addWidget(self.splitter)
        self.groupBox = QtWidgets.QGroupBox(parent=Dialog)
        self.groupBox.setTitle("Styling")
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.widgetsButton = QtWidgets.QPushButton(parent=self.groupBox)
        self.widgetsButton.setText("Qt Widget Gallery")
        self.widgetsButton.setObjectName("widgetsButton")
        self.verticalLayout_2.addWidget(self.widgetsButton)
        self.verticalLayout.addWidget(self.groupBox)

        self.retranslateUi(Dialog)
        self.script.setCurrentIndex(-1)
        QtCore.QMetaObject.connectSlotsByName(Dialog)
        Dialog.setTabOrder(self.text, self.widgetsButton)
        Dialog.setTabOrder(self.widgetsButton, self.script)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(tr.qt_misc_debug_console())


# Copyright: Ankitects Pty Ltd and contributors
# License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html

from anki.utils import is_mac, is_win
from aqt import colors, props
from aqt.theme import ThemeManager


def button_gradient(start: str, end: str) -> str:
    return f"""
qlineargradient(
    spread:pad, x1:0.5, y1:0, x2:0.5, y2:1,
    stop:0 {start},
    stop:1 {end}
);
    """


def button_pressed_gradient(start: str, end: str, shadow: str) -> str:
    return f"""
qlineargradient(
    spread:pad, x1:0.5, y1:0, x2:0.5, y2:1,
    stop:0 {shadow},
    stop:0.1 {start},
    stop:0.9 {end},
    stop:1 {shadow}
);
    """


def button_layout(tm: ThemeManager):
    # https://doc.qt.io/qt-6/stylesheet-reference.html#button-layout
    if is_win:
        return 0
    elif is_mac:
        return 1
    # on linux, use non-default layout if available
    if tm._default_button_layout:
        return tm._default_button_layout
    # fallback to GnomeLayout
    return 3


class CustomStyles:
    def general(self, tm: ThemeManager) -> str:
        return f"""
    QFrame,
    QWidget {{
        background: none;
    }}
    QPushButton,
    QComboBox,
    QSpinBox,
    QDateTimeEdit,
    QLineEdit,
    QListWidget,
    QTreeWidget,
    QListView,
    QTextEdit,
    QPlainTextEdit {{
        border: 1px solid {tm.var(colors.BORDER_SUBTLE)};
        border-radius: {tm.var(props.BORDER_RADIUS)};
    }}
    QLineEdit,
    QTextEdit,
    QPlainTextEdit,
    QDateTimeEdit,
    QListWidget,
    QTreeWidget,
    QListView {{
        background: {tm.var(colors.CANVAS_CODE)};
    }}
    QLineEdit,
    QTextEdit,
    QPlainTextEdit,
    QDateTimeEdit {{
        padding: 2px;
    }}
    QSpinBox:focus,
    QDateTimeEdit:focus,
    QLineEdit:focus,
    QTextEdit:editable:focus,
    QPlainTextEdit:editable:focus,
    QWidget:editable:focus {{
        border-color: {tm.var(colors.BORDER_FOCUS)};
    }}
    QPushButton {{
        margin-top: 1px;
    }}
    QPushButton,
    QComboBox,
    QSpinBox {{
        padding: 2px 6px;
    }}
    QGroupBox {{
        text-align: center;
        font-weight: bold;
        border: 1px solid {tm.var(colors.BORDER_SUBTLE)};
        padding: 0.75em 0 0.75em 0;
        background: {tm.var(colors.CANVAS_ELEVATED)};
        border-radius: {tm.var(props.BORDER_RADIUS)};
        margin-top: 10px;
    }}
    QGroupBox#preview_box,
    QGroupBox#template_box {{
        background: none;
        border: none;
    }}
    QGroupBox::title {{
        subcontrol-origin: margin;
        subcontrol-position: top left;
        margin: 0 2px;
        left: 15px;
    }}
    QGroupBox#preview_box::title,
    QGroupBox#template_box::title {{
        margin-top: 5px;
        left: 5px;
    }}
    QLabel:disabled {{
        color: {tm.var(colors.FG_DISABLED)};
    }}
    QToolTip {{ color: {tm.var(colors.FG)}; background-color: {tm.var(colors.CANVAS)}; }}
        """

    def menu(self, tm: ThemeManager) -> str:
        return f"""
    QMenuBar {{
        border-bottom: 1px solid {tm.var(colors.BORDER_SUBTLE)};
    }}
    QMenuBar::item {{
        background-color: transparent;
        padding: 2px 4px;
        border-radius: {tm.var(props.BORDER_RADIUS)};
    }}
    QMenuBar::item:selected {{
        background-color: {tm.var(colors.CANVAS_ELEVATED)};
    }}
    QMenu {{
        background-color: {tm.var(colors.CANVAS_OVERLAY)};
        border: 1px solid {tm.var(colors.BORDER_SUBTLE)};
        padding: 4px;
    }}
    QMenu::item {{
        background-color: transparent;
        padding: 3px 14px;
        margin-bottom: 4px;
    }}
    QMenu::item:selected {{
        background-color: {tm.var(colors.HIGHLIGHT_BG)};
        border-radius: {tm.var(props.BORDER_RADIUS)};
    }}
    QMenu::separator {{
        height: 1px;
        background: {tm.var(colors.BORDER_SUBTLE)};
        margin: 0 8px 4px 8px;
    }}
    QMenu::indicator {{
        border: 1px solid {tm.var(colors.BORDER)};
        margin-{tm.left()}: 6px;
        margin-{tm.right()}: -6px;
    }}
        """

    def button(self, tm: ThemeManager) -> str:
        # For some reason, Windows needs a larger padding to look the same
        button_pad = 25 if is_win else 15
        return f"""
    QPushButton {{ padding-left: {button_pad}px; padding-right: {button_pad}px; }}
    QPushButton,
    QTabBar::tab:!selected,
    QComboBox:!editable,
    QComboBox::drop-down:editable {{
        background: {tm.var(colors.BUTTON_BG)};
        border-bottom: 1px solid {tm.var(colors.SHADOW)};
    }}
    QPushButton:default {{
        border: 1px solid {tm.var(colors.BORDER_FOCUS)};
    }}
    QPushButton {{
        margin: 1px;
    }}
    QPushButton:focus, QPushButton:default:hover {{
        border: 2px solid {tm.var(colors.BORDER_FOCUS)};
        outline: none;
        margin: 0px;
    }}
    QPushButton:hover,
    QTabBar::tab:hover,
    QComboBox:!editable:hover,
    QSpinBox::up-button:hover,
    QSpinBox::down-button:hover,
    QDateTimeEdit::up-button:hover,
    QDateTimeEdit::down-button:hover {{
        background: {
            button_gradient(
                tm.var(colors.BUTTON_GRADIENT_START),
                tm.var(colors.BUTTON_GRADIENT_END),
            )
        };
    }}
    QPushButton:pressed,
    QPushButton:checked,
    QSpinBox::up-button:pressed,
    QSpinBox::down-button:pressed,
    QDateTimeEdit::up-button:pressed,
    QDateTimeEdit::down-button:pressed {{
        background: {
            button_pressed_gradient(
                tm.var(colors.BUTTON_GRADIENT_START),
                tm.var(colors.BUTTON_GRADIENT_END),
                tm.var(colors.SHADOW),
            )
        };
    }}
    QPushButton:flat {{
        border: none;
    }}
    QDialogButtonBox {{
        button-layout: {button_layout(tm)};
    }}
        """

    def splitter(self, tm: ThemeManager) -> str:
        return f"""
    QSplitter::handle,
    QMainWindow::separator {{
        height: 16px;
    }}
    QSplitter::handle:vertical,
    QMainWindow::separator:horizontal {{
        image: url({tm.themed_icon("mdi:drag-horizontal-FG_SUBTLE")});
    }}
    QSplitter::handle:horizontal,
    QMainWindow::separator:vertical {{
        image: url({tm.themed_icon("mdi:drag-vertical-FG_SUBTLE")});
    }}
    """

    def combobox(self, tm: ThemeManager) -> str:
        return f"""
    QComboBox {{
        padding: {"1px 6px 2px 4px" if tm.rtl() else "1px 4px 2px 6px"};
    }}
    QComboBox:focus {{
        border-color: {tm.var(colors.BORDER_FOCUS)};
    }}
    QComboBox:editable:on,
    QComboBox:editable:focus,
    QComboBox::drop-down:focus:editable,
    QComboBox::drop-down:pressed {{
        border-color: {tm.var(colors.BORDER_FOCUS)};
    }}
    QComboBox:on {{
        border-bottom: none;
        border-bottom-right-radius: 0;
        border-bottom-left-radius: 0;
    }}
    QComboBox::item {{
        color: {tm.var(colors.FG)};
        background: {tm.var(colors.CANVAS_ELEVATED)};
    }}

    QComboBox::item:selected {{
        background: {tm.var(colors.HIGHLIGHT_BG)};
        color: {tm.var(colors.HIGHLIGHT_FG)};
    }}
    QComboBox::item::icon:selected {{
        position: absolute;
    }}
    QComboBox::drop-down {{
        subcontrol-origin: border;
        padding: 2px;
        padding-left: 4px;
        padding-right: 4px;
        width: 16px;
        subcontrol-position: top right;
        border: 1px solid {tm.var(colors.BORDER_SUBTLE)};
        border-top-{tm.right()}-radius: {tm.var(props.BORDER_RADIUS)};
        border-bottom-{tm.right()}-radius: {tm.var(props.BORDER_RADIUS)};
    }}
    QComboBox::drop-down:!editable {{
        background: none;
        border-color: transparent;
    }}
    QComboBox::down-arrow {{
        image: url({tm.themed_icon("mdi:chevron-down")});
    }}
    QComboBox::down-arrow:disabled {{
        image: url({tm.themed_icon("mdi:chevron-down-FG_DISABLED")});
    }}
    QComboBox::drop-down:hover:editable {{
        background: {
            button_gradient(
                tm.var(colors.BUTTON_GRADIENT_START),
                tm.var(colors.BUTTON_GRADIENT_END),
            )
        };
    }}
        """

    def tabwidget(self, tm: ThemeManager) -> str:
        return f"""
    QTabWidget {{
        border-radius: {tm.var(props.BORDER_RADIUS)};
        background: none;
    }}
    QTabWidget::pane {{
        top: -15px;
        padding-top: 1em;
        background: {tm.var(colors.CANVAS_ELEVATED)};
        border: 1px solid {tm.var(colors.BORDER_SUBTLE)};
        border-radius: {tm.var(props.BORDER_RADIUS)};
    }}
    QTabWidget::tab-bar {{
        alignment: center;
    }}
    QTabBar::tab {{
        background: none;
        padding: 4px 8px;
        min-width: 8ex;
    }}
    QTabBar::tab {{
        border: 1px solid {tm.var(colors.BORDER_SUBTLE)};
        border-bottom-color: {tm.var(colors.SHADOW)};
    }}
    QTabBar::tab:first {{
        border-top-{tm.left()}-radius: {tm.var(props.BORDER_RADIUS)};
        border-bottom-{tm.left()}-radius: {tm.var(props.BORDER_RADIUS)};
    }}
    QTabBar::tab:!first {{
        margin-{tm.left()}: -1px;
    }}
    QTabBar::tab:last {{
        border-top-{tm.right()}-radius: {tm.var(props.BORDER_RADIUS)};
        border-bottom-{tm.right()}-radius: {tm.var(props.BORDER_RADIUS)};
    }}
    QTabBar::tab:selected {{
        color: white;
        background: {tm.var(colors.BUTTON_PRIMARY_BG)};
    }}
    QTabBar::tab:selected:hover {{
        background: {
            button_gradient(
                tm.var(colors.BUTTON_PRIMARY_GRADIENT_START),
                tm.var(colors.BUTTON_PRIMARY_GRADIENT_END),
            )
        };
    }}
    QTabBar::tab:focus {{
        outline: none;
    }}
    QTabBar::tab:disabled,
    QTabBar::tab:disabled:hover {{
        background: {tm.var(colors.BUTTON_DISABLED)};
        color: {tm.var(colors.FG_DISABLED)};
    }}
    QTabBar::tab:selected:disabled,
    QTabBar::tab:selected:hover:disabled {{
        background: {tm.var(colors.BUTTON_PRIMARY_DISABLED)};
    }}
        """

    def table(self, tm: ThemeManager) -> str:
        return f"""
    QTableView {{
        border-radius: {tm.var(props.BORDER_RADIUS)};
        border-{tm.left()}: 1px solid {tm.var(colors.BORDER_SUBTLE)};
        border-bottom: 1px solid {tm.var(colors.BORDER_SUBTLE)};
        border-bottom-left-radius: 0;
        border-bottom-right-radius: 0;
        gridline-color: {tm.var(colors.BORDER_SUBTLE)};
        selection-background-color: {tm.var(colors.SELECTED_BG)};
        selection-color: {tm.var(colors.SELECTED_FG)};
        background: {tm.var(colors.CANVAS_CODE)};
    }}
    QHeaderView {{
        background: {tm.var(colors.CANVAS)};
    }}
    QHeaderView::section {{
        padding-{tm.left()}: 0px;
        padding-{tm.right()}: 15px;
        border: 1px solid {tm.var(colors.BORDER_SUBTLE)};
        background: {tm.var(colors.BUTTON_BG)};
    }}
    QHeaderView::section:first {{
        margin-left: -1px;
    }}
    QHeaderView::section:pressed,
    QHeaderView::section:pressed:!first {{
        background: {
            button_pressed_gradient(
                tm.var(colors.BUTTON_GRADIENT_START),
                tm.var(colors.BUTTON_GRADIENT_END),
                tm.var(colors.SHADOW),
            )
        }
    }}
    QHeaderView::section:hover {{
        background: {
            button_gradient(
                tm.var(colors.BUTTON_GRADIENT_START),
                tm.var(colors.BUTTON_GRADIENT_END),
            )
        };
    }}
    QHeaderView::section:first {{
        border-left: 1px solid {tm.var(colors.BORDER_SUBTLE)};
        border-top-left-radius: {tm.var(props.BORDER_RADIUS)};
    }}
    QHeaderView::section:!first {{
        border-left: none;
    }}
    QHeaderView::section:last {{
        border-right: 1px solid {tm.var(colors.BORDER_SUBTLE)};
        border-top-right-radius: {tm.var(props.BORDER_RADIUS)};
    }}
    QHeaderView::section:only-one {{
        border-left: 1px solid {tm.var(colors.BORDER_SUBTLE)};
        border-right: 1px solid {tm.var(colors.BORDER_SUBTLE)};
        border-top-left-radius: {tm.var(props.BORDER_RADIUS)};
        border-top-right-radius: {tm.var(props.BORDER_RADIUS)};
    }}
    QHeaderView::up-arrow,
    QHeaderView::down-arrow {{
        width: 20px;
        height: 20px;
        margin-{tm.left()}: -20px;
    }}
    QHeaderView::up-arrow {{
        image: url({tm.themed_icon("mdi:menu-up")});
    }}
    QHeaderView::down-arrow {{
        image: url({tm.themed_icon("mdi:menu-down")});
    }}
        """

    def spinbox(self, tm: ThemeManager) -> str:
        return f"""
    QSpinBox::up-button,
    QSpinBox::down-button,
    QDateTimeEdit::up-button,
    QDateTimeEdit::down-button {{
        subcontrol-origin: border;
        width: 16px;
        margin: 1px;
    }}
    QSpinBox::up-button,
    QDateTimeEdit::up-button {{
        margin-bottom: -1px;
        subcontrol-position: top right;
        border-top-{tm.right()}-radius: {tm.var(props.BORDER_RADIUS)};
    }}
    QSpinBox::down-button,
    QDateTimeEdit::down-button {{
        margin-top: -1px;
        subcontrol-position: bottom right;
        border-bottom-{tm.right()}-radius: {tm.var(props.BORDER_RADIUS)};
    }}
    QSpinBox::up-arrow,
    QDateTimeEdit::up-arrow {{
        image: url({tm.themed_icon("mdi:chevron-up")});
    }}
    QSpinBox::down-arrow,
    QDateTimeEdit::down-arrow {{
        image: url({tm.themed_icon("mdi:chevron-down")});
    }}
    QSpinBox::up-arrow,
    QSpinBox::down-arrow,
    QSpinBox::up-arrow:pressed,
    QSpinBox::down-arrow:pressed,
    QSpinBox::up-arrow:disabled:hover, QSpinBox::up-arrow:off:hover,
    QSpinBox::down-arrow:disabled:hover, QSpinBox::down-arrow:off:hover,
    QDateTimeEdit::up-arrow,
    QDateTimeEdit::down-arrow,
    QDateTimeEdit::up-arrow:pressed,
    QDateTimeEdit::down-arrow:pressed,
    QDateTimeEdit::up-arrow:disabled:hover, QDateTimeEdit::up-arrow:off:hover,
    QDateTimeEdit::down-arrow:disabled:hover, QDateTimeEdit::down-arrow:off:hover {{
        width: 16px;
        height: 16px;
    }}
    QSpinBox::up-arrow:hover,
    QSpinBox::down-arrow:hover,
    QDateTimeEdit::up-arrow:hover,
    QDateTimeEdit::down-arrow:hover {{
        width: 20px;
        height: 20px;
    }}
    QSpinBox::up-arrow:disabled, QSpinBox::up-arrow:off,
    QDateTimeEdit::up-arrow:disabled, QDateTimeEdit::up-arrow:off {{
        image: url({tm.themed_icon("mdi:chevron-up-FG_DISABLED")});
    }}
    QSpinBox::down-arrow:disabled, QSpinBox::down-arrow:off,
    QDateTimeEdit::down-arrow:disabled, QDateTimeEdit::down-arrow:off {{
        image: url({tm.themed_icon("mdi:chevron-down-FG_DISABLED")});
    }}
        """

    def checkbox(self, tm: ThemeManager) -> str:
        return f"""
    QCheckBox,
    QRadioButton {{
        spacing: 8px;
        margin: 2px 0;
    }}
    QCheckBox::indicator,
    QRadioButton::indicator,
    QMenu::indicator {{
        border: 1px solid {tm.var(colors.BORDER)};
        border-radius: {tm.var(props.BORDER_RADIUS)};
        background: {tm.var(colors.CANVAS_ELEVATED)};
        width: 16px;
        height: 16px;
    }}
    QRadioButton::indicator,
    QMenu::indicator:exclusive {{
        border-radius: 8px;
    }}
    QCheckBox::indicator:focus:!disabled,
    QCheckBox::indicator:hover:!disabled,
    QCheckBox::indicator:checked:hover:!disabled,
    QRadioButton::indicator:focus:!disabled,
    QRadioButton::indicator:hover:!disabled,
    QRadioButton::indicator:checked::!disabled {{
        border: 2px solid {tm.var(colors.BORDER_STRONG)};
        width: 14px;
        height: 14px;
    }}
    QCheckBox::indicator:checked,
    QRadioButton::indicator:checked,
    QMenu::indicator:checked {{
        image: url({tm.themed_icon("mdi:check")});
    }}
    QRadioButton::indicator:checked {{
        image: url({tm.themed_icon("mdi:circle-medium")});
    }}
    QCheckBox::indicator:indeterminate {{
        image: url({tm.themed_icon("mdi:minus-thick")});
    }}
    QCheckBox:disabled,
    QRadioButton:disabled {{
        color: {tm.var(colors.FG_DISABLED)};
    }}
    QCheckBox::indicator:disabled,
    QRadioButton::indicator:disabled,
    QMenu:indicator:disabled {{
        color: {tm.var(colors.FG_DISABLED)};
        border-color: {tm.var(colors.FG_DISABLED)};
    }}
    QCheckBox::indicator:checked:disabled,
    QRadioButton::indicator:checked:disabled,
    QMenu::indicator:checked:disabled {{
        image: url({tm.themed_icon("mdi:check-FG_DISABLED")});
    }}
    QRadioButton::indicator:checked:disabled {{
        image: url({tm.themed_icon("mdi:circle-medium-FG_DISABLED")});
    }}
    QCheckBox::indicator:indeterminate:disabled {{
        image: url({tm.themed_icon("mdi:minus-thick-FG_DISABLED")});
    }}
        """

    def scrollbar(self, tm: ThemeManager) -> str:
        return f"""
    QAbstractScrollArea::corner {{
        background: none;
        border: none;
    }}
    QScrollBar {{
        subcontrol-origin: content;
        background-color: transparent;
    }}
    QScrollBar::handle {{
        border-radius: {tm.var(props.BORDER_RADIUS)};
        background-color: {tm.var(colors.SCROLLBAR_BG)};
    }}
    QScrollBar::handle:hover {{
        background-color: {tm.var(colors.SCROLLBAR_BG_HOVER)};
    }}
    QScrollBar::handle:pressed {{
        background-color: {tm.var(colors.SCROLLBAR_BG_ACTIVE)};
    }}
    QScrollBar:horizontal {{
        height: 12px;
    }}
    QScrollBar::handle:horizontal {{
        min-width: 60px;
    }}
    QScrollBar:vertical {{
        width: 12px;
    }}
    QScrollBar::handle:vertical {{
        min-height: 60px;
    }}
    QScrollBar::add-line {{
        border: none;
        background: none;
    }}
    QScrollBar::sub-line {{
        border: none;
        background: none;
    }}
        """

    def slider(self, tm: ThemeManager) -> str:
        return f"""
    QSlider::horizontal {{
        height: 20px;
    }}
    QSlider::vertical {{
        width: 20px;
    }}
    QSlider::groove {{
        border: 1px solid {tm.var(colors.BORDER_SUBTLE)};
        border-radius: 3px;
        background: {tm.var(colors.CANVAS_ELEVATED)};
    }}
    QSlider::sub-page {{
        background: {tm.var(colors.BUTTON_PRIMARY_GRADIENT_START)};
        border-radius: 3px;
        margin: 1px;
    }}
    QSlider::sub-page:disabled {{
        background: {tm.var(colors.BUTTON_DISABLED)};
    }}
    QSlider::add-page {{
        margin-{tm.right()}: 2px;
    }}
    QSlider::groove:vertical {{
        width: 6px;
    }}
    QSlider::groove:horizontal {{
        height: 6px;
    }}
    QSlider::handle {{
        background: {tm.var(colors.BUTTON_BG)};
        border: 1px solid {tm.var(colors.BORDER)};
        border-radius: 9px;
        width: 18px;
        height: 18px;
        border-bottom-color: {tm.var(colors.SHADOW)};
    }}
    QSlider::handle:vertical {{
        margin: 0 -7px;
    }}
    QSlider::handle:horizontal {{
        margin: -7px 0;
    }}
    QSlider::handle:hover {{
        background: {
            button_gradient(
                tm.var(colors.BUTTON_GRADIENT_START),
                tm.var(colors.BUTTON_GRADIENT_END),
            )
        }
    }}
    """


custom_styles = CustomStyles()



# Copyright: Ankitects Pty Ltd and contributors
# License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html

from __future__ import annotations

import anki.scheduler.base as _base

UnburyDeck = _base.UnburyDeck
CongratsInfo = _base.CongratsInfo
BuryOrSuspend = _base.BuryOrSuspend
FilteredDeckForUpdate = _base.FilteredDeckForUpdate
CustomStudyRequest = _base.CustomStudyRequest



# from subtlepatterns.com; CC BY 4.0.
# by Daniel Beaton
# https://www.toptal.com/designers/subtlepatterns/fancy-deboss/
bg = """\
iVBORw0KGgoAAAANSUhEUgAAABIAAAANCAMAAACTkM4rAAAAM1BMVEXy8vLz8/P5+fn19fXt7e329vb4+Pj09PTv7+/u7u739/fw8PD7+/vx8fHr6+v6+vrs7Oz2LjW2AAAAkUlEQVR42g3KyXHAQAwDQYAQj12ItvOP1qqZZwMMPVnd06XToQvz4L2HDQ2iRgkvA7yPPB+JD+OUPnfzZ0JNZh6kkQus5NUmR7g4Jpxv5XN6nYWNmtlq9o3zuK6w3XRsE1pQIEGPIsdtTP3m2cYwlPv6MbL8/QASsKppZefyDmJPbxvxa/NrX1TJ1yp20fhj9D+SiAWWLU8myQAAAABJRU5ErkJggg==
"""



# Copyright: Ankitects Pty Ltd and contributors
# License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html

# This file is automatically generated; edit tools/genhooks.py instead.
# Please import from anki.hooks instead of this file.

# ruff: noqa: F401

from __future__ import annotations

from collections.abc import Callable, Sequence
from typing import Any

import anki
import anki.hooks
from anki.cards import Card
from anki.notes import Note

class _CardDidRenderHook:
    '''Can modify the resulting text after rendering completes.'''
    _hooks: list[Callable[["anki.template.TemplateRenderOutput", "anki.template.TemplateRenderContext"], None]] = []

    
    def append(self, callback: Callable[["anki.template.TemplateRenderOutput", "anki.template.TemplateRenderContext"], None]) -> None:
        '''(output: anki.template.TemplateRenderOutput, ctx: anki.template.TemplateRenderContext)'''
        self._hooks.append(callback)

    def remove(self, callback: Callable[["anki.template.TemplateRenderOutput", "anki.template.TemplateRenderContext"], None]) -> None:
        if callback in self._hooks:
            self._hooks.remove(callback)

    def count(self) -> int:
        return len(self._hooks)

    def __call__(self, output: anki.template.TemplateRenderOutput, ctx: anki.template.TemplateRenderContext) -> None:
        for hook in self._hooks:
            try:
                hook(output, ctx)
            except Exception:
                # if the hook fails, remove it
                self._hooks.remove(hook)
                raise



card_did_render = _CardDidRenderHook()
class _CardOdueWasInvalidHook:
    _hooks: list[Callable[[], None]] = []

    
    def append(self, callback: Callable[[], None]) -> None:
        '''()'''
        self._hooks.append(callback)

    def remove(self, callback: Callable[[], None]) -> None:
        if callback in self._hooks:
            self._hooks.remove(callback)

    def count(self) -> int:
        return len(self._hooks)

    def __call__(self) -> None:
        for hook in self._hooks:
            try:
                hook()
            except Exception:
                # if the hook fails, remove it
                self._hooks.remove(hook)
                raise



card_odue_was_invalid = _CardOdueWasInvalidHook()
class _CardWillFlushHook:
    '''Allow to change a card before it is added/updated in the database.'''
    _hooks: list[Callable[["Card"], None]] = []

    
    def append(self, callback: Callable[["Card"], None]) -> None:
        '''(card: Card)'''
        self._hooks.append(callback)

    def remove(self, callback: Callable[["Card"], None]) -> None:
        if callback in self._hooks:
            self._hooks.remove(callback)

    def count(self) -> int:
        return len(self._hooks)

    def __call__(self, card: Card) -> None:
        for hook in self._hooks:
            try:
                hook(card)
            except Exception:
                # if the hook fails, remove it
                self._hooks.remove(hook)
                raise



card_will_flush = _CardWillFlushHook()
class _DeckAddedHook:
    '''Obsolete, do not use.'''
    _hooks: list[Callable[["anki.decks.DeckDict"], None]] = []

    
    def append(self, callback: Callable[["anki.decks.DeckDict"], None]) -> None:
        '''(deck: anki.decks.DeckDict)'''
        self._hooks.append(callback)

    def remove(self, callback: Callable[["anki.decks.DeckDict"], None]) -> None:
        if callback in self._hooks:
            self._hooks.remove(callback)

    def count(self) -> int:
        return len(self._hooks)

    def __call__(self, deck: anki.decks.DeckDict) -> None:
        for hook in self._hooks:
            try:
                hook(deck)
            except Exception:
                # if the hook fails, remove it
                self._hooks.remove(hook)
                raise



deck_added = _DeckAddedHook()
class _ExportersListCreatedHook:
    _hooks: list[Callable[["list[tuple[str, Any]]"], None]] = []

    
    def append(self, callback: Callable[["list[tuple[str, Any]]"], None]) -> None:
        '''(exporters: list[tuple[str, Any]])'''
        self._hooks.append(callback)

    def remove(self, callback: Callable[["list[tuple[str, Any]]"], None]) -> None:
        if callback in self._hooks:
            self._hooks.remove(callback)

    def count(self) -> int:
        return len(self._hooks)

    def __call__(self, exporters: list[tuple[str, Any]]) -> None:
        for hook in self._hooks:
            try:
                hook(exporters)
            except Exception:
                # if the hook fails, remove it
                self._hooks.remove(hook)
                raise
        # legacy support
        anki.hooks.runHook("exportersList", exporters)



exporters_list_created = _ExportersListCreatedHook()
class _FieldFilterFilter:
    '''Allows you to define custom {{filters:..}}
        
        Your add-on can check filter_name to decide whether it should modify
        field_text or not before returning it.'''
    _hooks: list[Callable[["str", "str", "str", "anki.template.TemplateRenderContext"], str]] = []

    
    def append(self, callback: Callable[["str", "str", "str", "anki.template.TemplateRenderContext"], str]) -> None:
        '''(field_text: str, field_name: str, filter_name: str, ctx: anki.template.TemplateRenderContext)'''
        self._hooks.append(callback)

    def remove(self, callback: Callable[["str", "str", "str", "anki.template.TemplateRenderContext"], str]) -> None:
        if callback in self._hooks:
            self._hooks.remove(callback)

    def count(self) -> int:
        return len(self._hooks)

    def __call__(self, field_text: str, field_name: str, filter_name: str, ctx: anki.template.TemplateRenderContext) -> str:
        for filter in self._hooks:
            try:
                field_text = filter(field_text, field_name, filter_name, ctx)
            except Exception:
                # if the hook fails, remove it
                self._hooks.remove(filter)
                raise
        return field_text



field_filter = _FieldFilterFilter()
class _ImportingImportersHook:
    '''Allows updating the list of importers.
        The resulting list is not saved and should be changed each time the
        filter is called.
        
        NOTE: Updates to the import/export code are expected in the coming 
        months, and this hook may be replaced with another solution at that 
        time. Tracked on https://github.com/ankitects/anki/issues/1018'''
    _hooks: list[Callable[["list[tuple[str, Any]]"], None]] = []

    
    def append(self, callback: Callable[["list[tuple[str, Any]]"], None]) -> None:
        '''(importers: list[tuple[str, Any]])'''
        self._hooks.append(callback)

    def remove(self, callback: Callable[["list[tuple[str, Any]]"], None]) -> None:
        if callback in self._hooks:
            self._hooks.remove(callback)

    def count(self) -> int:
        return len(self._hooks)

    def __call__(self, importers: list[tuple[str, Any]]) -> None:
        for hook in self._hooks:
            try:
                hook(importers)
            except Exception:
                # if the hook fails, remove it
                self._hooks.remove(hook)
                raise



importing_importers = _ImportingImportersHook()
class _LegacyExportProgressHook:
    '''Temporary hook used in transition to new import/export code.'''
    _hooks: list[Callable[["str"], None]] = []

    
    def append(self, callback: Callable[["str"], None]) -> None:
        '''(progress: str)'''
        self._hooks.append(callback)

    def remove(self, callback: Callable[["str"], None]) -> None:
        if callback in self._hooks:
            self._hooks.remove(callback)

    def count(self) -> int:
        return len(self._hooks)

    def __call__(self, progress: str) -> None:
        for hook in self._hooks:
            try:
                hook(progress)
            except Exception:
                # if the hook fails, remove it
                self._hooks.remove(hook)
                raise



legacy_export_progress = _LegacyExportProgressHook()
class _MediaFileFilterFilter:
    '''Allows manipulating the file path that media will be read from'''
    _hooks: list[Callable[["str"], str]] = []

    
    def append(self, callback: Callable[["str"], str]) -> None:
        '''(txt: str)'''
        self._hooks.append(callback)

    def remove(self, callback: Callable[["str"], str]) -> None:
        if callback in self._hooks:
            self._hooks.remove(callback)

    def count(self) -> int:
        return len(self._hooks)

    def __call__(self, txt: str) -> str:
        for filter in self._hooks:
            try:
                txt = filter(txt)
            except Exception:
                # if the hook fails, remove it
                self._hooks.remove(filter)
                raise
        return txt



media_file_filter = _MediaFileFilterFilter()
class _MediaFilesDidExportHook:
    '''Only used by legacy .apkg exporter. Will be deprecated in the future.'''
    _hooks: list[Callable[["int"], None]] = []

    
    def append(self, callback: Callable[["int"], None]) -> None:
        '''(count: int)'''
        self._hooks.append(callback)

    def remove(self, callback: Callable[["int"], None]) -> None:
        if callback in self._hooks:
            self._hooks.remove(callback)

    def count(self) -> int:
        return len(self._hooks)

    def __call__(self, count: int) -> None:
        for hook in self._hooks:
            try:
                hook(count)
            except Exception:
                # if the hook fails, remove it
                self._hooks.remove(hook)
                raise



media_files_did_export = _MediaFilesDidExportHook()
class _NoteTypeAddedHook:
    '''Obsolete, do not use.'''
    _hooks: list[Callable[["anki.models.NotetypeDict"], None]] = []

    
    def append(self, callback: Callable[["anki.models.NotetypeDict"], None]) -> None:
        '''(notetype: anki.models.NotetypeDict)'''
        self._hooks.append(callback)

    def remove(self, callback: Callable[["anki.models.NotetypeDict"], None]) -> None:
        if callback in self._hooks:
            self._hooks.remove(callback)

    def count(self) -> int:
        return len(self._hooks)

    def __call__(self, notetype: anki.models.NotetypeDict) -> None:
        for hook in self._hooks:
            try:
                hook(notetype)
            except Exception:
                # if the hook fails, remove it
                self._hooks.remove(hook)
                raise



note_type_added = _NoteTypeAddedHook()
class _NoteWillBeAddedHook:
    '''Allows modifying a note before it's added to the collection.

        This hook may be called both when users use the Add screen, and when
        add-ons like AnkiConnect add notes. It is not called when importing. If
        you wish to alter the Add screen, use gui_hooks.add_cards_will_add_note
        instead.'''
    _hooks: list[Callable[["anki.collection.Collection", "anki.notes.Note", "anki.decks.DeckId"], None]] = []

    
    def append(self, callback: Callable[["anki.collection.Collection", "anki.notes.Note", "anki.decks.DeckId"], None]) -> None:
        '''(col: anki.collection.Collection, note: anki.notes.Note, deck_id: anki.decks.DeckId)'''
        self._hooks.append(callback)

    def remove(self, callback: Callable[["anki.collection.Collection", "anki.notes.Note", "anki.decks.DeckId"], None]) -> None:
        if callback in self._hooks:
            self._hooks.remove(callback)

    def count(self) -> int:
        return len(self._hooks)

    def __call__(self, col: anki.collection.Collection, note: anki.notes.Note, deck_id: anki.decks.DeckId) -> None:
        for hook in self._hooks:
            try:
                hook(col, note, deck_id)
            except Exception:
                # if the hook fails, remove it
                self._hooks.remove(hook)
                raise



note_will_be_added = _NoteWillBeAddedHook()
class _NoteWillFlushHook:
    '''Allow to change a note before it is added/updated in the database.'''
    _hooks: list[Callable[["Note"], None]] = []

    
    def append(self, callback: Callable[["Note"], None]) -> None:
        '''(note: Note)'''
        self._hooks.append(callback)

    def remove(self, callback: Callable[["Note"], None]) -> None:
        if callback in self._hooks:
            self._hooks.remove(callback)

    def count(self) -> int:
        return len(self._hooks)

    def __call__(self, note: Note) -> None:
        for hook in self._hooks:
            try:
                hook(note)
            except Exception:
                # if the hook fails, remove it
                self._hooks.remove(hook)
                raise



note_will_flush = _NoteWillFlushHook()
class _NotesWillBeDeletedHook:
    _hooks: list[Callable[["anki.collection.Collection", "Sequence[anki.notes.NoteId]"], None]] = []

    
    def append(self, callback: Callable[["anki.collection.Collection", "Sequence[anki.notes.NoteId]"], None]) -> None:
        '''(col: anki.collection.Collection, ids: Sequence[anki.notes.NoteId])'''
        self._hooks.append(callback)

    def remove(self, callback: Callable[["anki.collection.Collection", "Sequence[anki.notes.NoteId]"], None]) -> None:
        if callback in self._hooks:
            self._hooks.remove(callback)

    def count(self) -> int:
        return len(self._hooks)

    def __call__(self, col: anki.collection.Collection, ids: Sequence[anki.notes.NoteId]) -> None:
        for hook in self._hooks:
            try:
                hook(col, ids)
            except Exception:
                # if the hook fails, remove it
                self._hooks.remove(hook)
                raise
        # legacy support
        anki.hooks.runHook("remNotes", col, ids)



notes_will_be_deleted = _NotesWillBeDeletedHook()
class _SchemaWillChangeFilter:
    _hooks: list[Callable[["bool"], bool]] = []

    
    def append(self, callback: Callable[["bool"], bool]) -> None:
        '''(proceed: bool)'''
        self._hooks.append(callback)

    def remove(self, callback: Callable[["bool"], bool]) -> None:
        if callback in self._hooks:
            self._hooks.remove(callback)

    def count(self) -> int:
        return len(self._hooks)

    def __call__(self, proceed: bool) -> bool:
        for filter in self._hooks:
            try:
                proceed = filter(proceed)
            except Exception:
                # if the hook fails, remove it
                self._hooks.remove(filter)
                raise
        return proceed



schema_will_change = _SchemaWillChangeFilter()
class _SyncProgressDidChangeHook:
    '''Obsolete, do not use.'''
    _hooks: list[Callable[["str"], None]] = []

    
    def append(self, callback: Callable[["str"], None]) -> None:
        '''(msg: str)'''
        self._hooks.append(callback)

    def remove(self, callback: Callable[["str"], None]) -> None:
        if callback in self._hooks:
            self._hooks.remove(callback)

    def count(self) -> int:
        return len(self._hooks)

    def __call__(self, msg: str) -> None:
        for hook in self._hooks:
            try:
                hook(msg)
            except Exception:
                # if the hook fails, remove it
                self._hooks.remove(hook)
                raise



sync_progress_did_change = _SyncProgressDidChangeHook()
class _SyncStageDidChangeHook:
    '''Obsolete, do not use.'''
    _hooks: list[Callable[["str"], None]] = []

    
    def append(self, callback: Callable[["str"], None]) -> None:
        '''(stage: str)'''
        self._hooks.append(callback)

    def remove(self, callback: Callable[["str"], None]) -> None:
        if callback in self._hooks:
            self._hooks.remove(callback)

    def count(self) -> int:
        return len(self._hooks)

    def __call__(self, stage: str) -> None:
        for hook in self._hooks:
            try:
                hook(stage)
            except Exception:
                # if the hook fails, remove it
                self._hooks.remove(hook)
                raise



sync_stage_did_change = _SyncStageDidChangeHook()




# Form implementation generated from reading ui file 'qt/aqt/forms/browseropts.ui'
#
# Created by: PyQt6 UI code generator 6.11.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets
from aqt.utils import tr



class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(288, 195)
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(parent=Dialog)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.fontCombo = QtWidgets.QFontComboBox(parent=Dialog)
        self.fontCombo.setObjectName("fontCombo")
        self.horizontalLayout.addWidget(self.fontCombo)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.label_2 = QtWidgets.QLabel(parent=Dialog)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 0, 0, 1, 1)
        self.fontSize = QtWidgets.QSpinBox(parent=Dialog)
        self.fontSize.setMinimumSize(QtCore.QSize(75, 0))
        self.fontSize.setObjectName("fontSize")
        self.gridLayout.addWidget(self.fontSize, 0, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(parent=Dialog)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 1, 0, 1, 1)
        self.lineSize = QtWidgets.QSpinBox(parent=Dialog)
        self.lineSize.setObjectName("lineSize")
        self.gridLayout.addWidget(self.lineSize, 1, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout.addItem(spacerItem, 0, 2, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.fullSearch = QtWidgets.QCheckBox(parent=Dialog)
        self.fullSearch.setObjectName("fullSearch")
        self.verticalLayout.addWidget(self.fullSearch)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout.addItem(spacerItem1)
        self.buttonBox = QtWidgets.QDialogButtonBox(parent=Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Cancel|QtWidgets.QDialogButtonBox.StandardButton.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(Dialog)
        self.buttonBox.accepted.connect(Dialog.accept) # type: ignore  # type: ignore
        self.buttonBox.rejected.connect(Dialog.reject) # type: ignore  # type: ignore
        QtCore.QMetaObject.connectSlotsByName(Dialog)
        Dialog.setTabOrder(self.fontCombo, self.fontSize)
        Dialog.setTabOrder(self.fontSize, self.lineSize)
        Dialog.setTabOrder(self.lineSize, self.fullSearch)
        Dialog.setTabOrder(self.fullSearch, self.buttonBox)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(tr.browsing_browser_options())
        self.label.setText(tr.browsing_font())
        self.label_2.setText(tr.browsing_font_size())
        self.label_3.setText(tr.browsing_line_size())
        self.fullSearch.setText(tr.browsing_search_within_formatting_slow())